@@ -66,7 +66,7 @@ void
 KeyDown(kbutton_t *b)
 {
     int k;
-    char *c;
+    const char *c;
 
     c = Cmd_Argv(1);
     if (c[0])
@@ -95,7 +95,7 @@ void
 KeyUp(kbutton_t *b)
 {
     int k;
-    char *c;
+    const char *c;
 
     c = Cmd_Argv(1);
     if (c[0])
@@ -137,13 +137,15 @@ void
 IN_MLookDown(void)
 {
     KeyDown(&in_mlook);
+    if (!((in_mlook.state & 1) ^ (int)m_freelook.value) && lookspring.value)
+	V_StartPitchDrift();
 }
 
 void
 IN_MLookUp(void)
 {
     KeyUp(&in_mlook);
-    if (!(in_mlook.state & 1) && lookspring.value)
+    if (!((in_mlook.state & 1) ^ (int)m_freelook.value) && lookspring.value)
 	V_StartPitchDrift();
 }
 
@@ -391,8 +393,8 @@ CL_KeyState(kbutton_t *key)
 //==========================================================================
 
 cvar_t cl_upspeed = { "cl_upspeed", "200" };
-cvar_t cl_forwardspeed = { "cl_forwardspeed", "200", true };
-cvar_t cl_backspeed = { "cl_backspeed", "200", true };
+cvar_t cl_forwardspeed = { "cl_forwardspeed", "200" };
+cvar_t cl_backspeed = { "cl_backspeed", "200" };
 cvar_t cl_sidespeed = { "cl_sidespeed", "350" };
 
 cvar_t cl_movespeedkey = { "cl_movespeedkey", "2.0" };
@@ -402,6 +404,8 @@ cvar_t cl_pitchspeed = { "cl_pitchspeed", "150" };
 
 cvar_t cl_anglespeedkey = { "cl_anglespeedkey", "1.5" };
 
+cvar_t cl_run = { "cl_run", "0", true };
+
 
 /*
 ================
@@ -416,7 +420,7 @@ CL_AdjustAngles(void)
     float speed;
     float up, down;
 
-    if (in_speed.state & 1)
+    if ((in_speed.state & 1) ^ (int)cl_run.value)
 	speed = host_frametime * cl_anglespeedkey.value;
     else
 	speed = host_frametime;
@@ -492,7 +496,7 @@ CL_BaseMove(usercmd_t *cmd)
 //
 // adjust for speed key
 //
-    if (in_speed.state & 1) {
+    if ((in_speed.state & 1) ^ (int)cl_run.value) {
 	cmd->forwardmove *= cl_movespeedkey.value;
 	cmd->sidemove *= cl_movespeedkey.value;
 	cmd->upmove *= cl_movespeedkey.value;
@@ -527,7 +531,10 @@ CL_SendMove(usercmd_t *cmd)
     MSG_WriteFloat(&buf, cl.mtime[0]);	/* so server can get ping times */
 
     for (i = 0; i < 3; i++)
-	MSG_WriteAngle(&buf, cl.viewangles[i]);
+	if (cl.protocol == PROTOCOL_VERSION_FITZ)
+	    MSG_WriteAngle16(&buf, cl.viewangles[i]);
+	else
+	    MSG_WriteAngle(&buf, cl.viewangles[i]);
 
     MSG_WriteShort(&buf, cmd->forwardmove);
     MSG_WriteShort(&buf, cmd->sidemove);
@@ -558,7 +565,7 @@ CL_SendMove(usercmd_t *cmd)
      */
 
     /*
-     * allways dump the first two message, because it may contain leftover
+     * always dump the first two message, because it may contain leftover
      * inputs from the last level
      */
     if (++cl.movemessages <= 2)
