@@ -27,8 +27,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "input.h"
 #include "keys.h"
 #include "menu.h"
+#include "model.h"
 #include "net.h"
-#include "net_vcr.h"
 #include "protocol.h"
 #include "quakedef.h"
 #include "sbar.h"
@@ -40,13 +40,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "wad.h"
 
 #ifdef GLQUAKE
-#include "gl_model.h"
+#include "glquake.h"
 #else
 #include "r_local.h"
+#include "render.h"
 #endif
 
 /*
- * A server can allways be started, even if the system started out as a client
+ * A server can always be started, even if the system started out as a client
  * to a remote system.
  *
  * A client can NOT be started if the system started as a dedicated server.
@@ -55,6 +56,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * end.
  */
 
+wad_t host_gfx; /* "gfx.wad" */
+
 quakeparms_t host_parms;
 
 qboolean host_initialized;	// true if into command execution
@@ -62,16 +65,16 @@ qboolean host_initialized;	// true if into command execution
 double host_frametime;
 double host_time;
 double realtime;		// without any filtering or bounding
-double oldrealtime;		// last frame run
+static double oldrealtime;	// last frame run
 int host_framecount;
 
 int host_hunklevel;
 
 int minimum_memory;
 
-client_t *host_client;		// current client
+int fps_count;
 
-jmp_buf host_abortserver;
+static jmp_buf host_abort;
 
 byte *host_basepal;
 byte *host_colormap;
@@ -106,13 +109,13 @@ Host_EndGame
 ================
 */
 void
-Host_EndGame(char *message, ...)
+Host_EndGame(const char *message, ...)
 {
     va_list argptr;
-    char string[1024];
+    char string[MAX_PRINTMSG];
 
     va_start(argptr, message);
-    vsprintf(string, message, argptr);
+    vsnprintf(string, sizeof(string), message, argptr);
     va_end(argptr);
     Con_DPrintf("%s: %s\n", __func__, string);
 
@@ -127,7 +130,7 @@ Host_EndGame(char *message, ...)
     else
 	CL_Disconnect();
 
-    longjmp(host_abortserver, 1);
+    longjmp(host_abort, 1);
 }
 
 /*
@@ -138,10 +141,10 @@ This shuts down both the client and server
 ================
 */
 void
-Host_Error(char *error, ...)
+Host_Error(const char *error, ...)
 {
     va_list argptr;
-    char string[1024];
+    char string[MAX_PRINTMSG];
     static qboolean inerror = false;
 
     if (inerror)
@@ -151,7 +154,7 @@ Host_Error(char *error, ...)
     SCR_EndLoadingPlaque();	// reenable screen updates
 
     va_start(argptr, error);
-    vsprintf(string, error, argptr);
+    vsnprintf(string, sizeof(string), error, argptr);
     va_end(argptr);
     Con_Printf("%s: %s\n", __func__, string);
 
@@ -166,7 +169,7 @@ Host_Error(char *error, ...)
 
     inerror = false;
 
-    longjmp(host_abortserver, 1);
+    longjmp(host_abort, 1);
 }
 
 /*
@@ -240,7 +243,6 @@ Host_InitLocal(void)
     Cvar_RegisterVariable(&samelevel);
     Cvar_RegisterVariable(&noexit);
     Cvar_RegisterVariable(&skill);
-    Cvar_RegisterVariable(&developer);
     Cvar_RegisterVariable(&deathmatch);
     Cvar_RegisterVariable(&coop);
 
@@ -248,6 +250,10 @@ Host_InitLocal(void)
 
     Cvar_RegisterVariable(&temp1);
 
+    Cvar_RegisterVariable(&developer);
+    if (COM_CheckParm("-developer"))
+	Cvar_SetValue("developer", 1);
+
     Host_FindMaxClients();
 
     host_time = 1.0;		// so a think at time 0 won't get called
@@ -278,10 +284,6 @@ Host_WriteConfiguration(void)
 	Key_WriteBindings(f);
 	Cvar_WriteVariables(f);
 
-	/* Save the mlook state (rarely used as an actual key binding) */
-	if (in_mlook.state & 1)
-	    fprintf(f, "+mlook\n");
-
 	fclose(f);
     }
 }
@@ -296,17 +298,14 @@ FIXME: make this just a stuffed echo?
 =================
 */
 void
-SV_ClientPrintf(char *fmt, ...)
+SV_ClientPrintf(client_t *client, const char *fmt, ...)
 {
     va_list argptr;
-    char string[1024];
 
+    MSG_WriteByte(&client->message, svc_print);
     va_start(argptr, fmt);
-    vsprintf(string, fmt, argptr);
+    MSG_WriteStringvf(&client->message, fmt, argptr);
     va_end(argptr);
-
-    MSG_WriteByte(&host_client->message, svc_print);
-    MSG_WriteString(&host_client->message, string);
 }
 
 /*
@@ -317,20 +316,17 @@ Sends text to all active clients
 =================
 */
 void
-SV_BroadcastPrintf(char *fmt, ...)
+SV_BroadcastPrintf(const char *fmt, ...)
 {
     va_list argptr;
-    char string[1024];
     int i;
 
-    va_start(argptr, fmt);
-    vsprintf(string, fmt, argptr);
-    va_end(argptr);
-
     for (i = 0; i < svs.maxclients; i++)
 	if (svs.clients[i].active && svs.clients[i].spawned) {
 	    MSG_WriteByte(&svs.clients[i].message, svc_print);
-	    MSG_WriteString(&svs.clients[i].message, string);
+	    va_start(argptr, fmt);
+	    MSG_WriteStringvf(&svs.clients[i].message, fmt, argptr);
+	    va_end(argptr);
 	}
 }
 
@@ -342,17 +338,14 @@ Send text over to the client to be executed
 =================
 */
 void
-Host_ClientCommands(char *fmt, ...)
+Host_ClientCommands(client_t *client, const char *fmt, ...)
 {
     va_list argptr;
-    char string[1024];
 
+    MSG_WriteByte(&client->message, svc_stufftext);
     va_start(argptr, fmt);
-    vsprintf(string, fmt, argptr);
+    MSG_WriteStringvf(&client->message, fmt, argptr);
     va_end(argptr);
-
-    MSG_WriteByte(&host_client->message, svc_stufftext);
-    MSG_WriteString(&host_client->message, string);
 }
 
 /*
@@ -364,54 +357,54 @@ if (crash = true), don't bother sending signofs
 =====================
 */
 void
-SV_DropClient(qboolean crash)
+SV_DropClient(client_t *client, qboolean crash)
 {
-    int saveSelf;
     int i;
-    client_t *client;
+    client_t *notify;
 
     if (!crash) {
-	// send any final messages (don't check for errors)
-	if (NET_CanSendMessage(host_client->netconnection)) {
-	    MSG_WriteByte(&host_client->message, svc_disconnect);
-	    NET_SendMessage(host_client->netconnection,
-			    &host_client->message);
+	/* send any final messages (don't check for errors) */
+	if (NET_CanSendMessage(client->netconnection)) {
+	    MSG_WriteByte(&client->message, svc_disconnect);
+	    NET_SendMessage(client->netconnection, &client->message);
 	}
-
-	if (host_client->edict && host_client->spawned) {
-	    // call the prog function for removing a client
-	    // this will set the body to a dead frame, among other things
-	    saveSelf = pr_global_struct->self;
-	    pr_global_struct->self = EDICT_TO_PROG(host_client->edict);
+	if (client->edict && client->spawned) {
+	    /*
+	     * call the prog function for removing a client
+	     * this will set the body to a dead frame, among other things
+	     */
+	    const int save_self = pr_global_struct->self;
+	    pr_global_struct->self = EDICT_TO_PROG(client->edict);
 	    PR_ExecuteProgram(pr_global_struct->ClientDisconnect);
-	    pr_global_struct->self = saveSelf;
+	    pr_global_struct->self = save_self;
 	}
-
-	Sys_Printf("Client %s removed\n", host_client->name);
+	Sys_Printf("Client %s removed\n", client->name);
     }
-// break the net connection
-    NET_Close(host_client->netconnection);
-    host_client->netconnection = NULL;
 
-// free the client (the body stays around)
-    host_client->active = false;
-    host_client->name[0] = 0;
-    host_client->old_frags = -999999;
+    /* break the net connection */
+    NET_Close(client->netconnection);
+    client->netconnection = NULL;
+
+    /* free the client (the body stays around) */
+    client->active = false;
+    client->name[0] = 0;
+    client->old_frags = -999999;
     net_activeconnections--;
 
-// send notification to all clients
-    for (i = 0, client = svs.clients; i < svs.maxclients; i++, client++) {
+    /* send notification to all clients */
+    notify = svs.clients;
+    for (i = 0; i < svs.maxclients; i++, notify++) {
 	if (!client->active)
 	    continue;
-	MSG_WriteByte(&client->message, svc_updatename);
-	MSG_WriteByte(&client->message, host_client - svs.clients);
-	MSG_WriteString(&client->message, "");
-	MSG_WriteByte(&client->message, svc_updatefrags);
-	MSG_WriteByte(&client->message, host_client - svs.clients);
-	MSG_WriteShort(&client->message, 0);
-	MSG_WriteByte(&client->message, svc_updatecolors);
-	MSG_WriteByte(&client->message, host_client - svs.clients);
-	MSG_WriteByte(&client->message, 0);
+	MSG_WriteByte(&notify->message, svc_updatename);
+	MSG_WriteByte(&notify->message, client - svs.clients);
+	MSG_WriteString(&notify->message, "");
+	MSG_WriteByte(&notify->message, svc_updatefrags);
+	MSG_WriteByte(&notify->message, client - svs.clients);
+	MSG_WriteShort(&notify->message, 0);
+	MSG_WriteByte(&notify->message, svc_updatecolors);
+	MSG_WriteByte(&notify->message, client - svs.clients);
+	MSG_WriteByte(&notify->message, 0);
     }
 }
 
@@ -425,8 +418,8 @@ This only happens at the end of a game, not between levels
 void
 Host_ShutdownServer(qboolean crash)
 {
-    int i;
-    int count;
+    client_t *client;
+    int i, count;
     sizebuf_t buf;
     byte message[4];
     double start;
@@ -436,51 +429,46 @@ Host_ShutdownServer(qboolean crash)
 
     sv.active = false;
 
-// stop all client sounds immediately
+    /* stop all client sounds immediately */
     if (cls.state >= ca_connected)
 	CL_Disconnect();
 
-// flush any pending messages - like the score!!!
+    /* flush any pending messages - like the score!!! */
     start = Sys_DoubleTime();
     do {
 	count = 0;
-	for (i = 0, host_client = svs.clients; i < svs.maxclients;
-	     i++, host_client++) {
-	    if (host_client->active && host_client->message.cursize) {
-		if (NET_CanSendMessage(host_client->netconnection)) {
-		    NET_SendMessage(host_client->netconnection,
-				    &host_client->message);
-		    SZ_Clear(&host_client->message);
-		} else {
-		    NET_GetMessage(host_client->netconnection);
-		    count++;
-		}
+	client = svs.clients;
+	for (i = 0; i < svs.maxclients; i++, client++) {
+	    if (!client->active || !client->message.cursize)
+		continue;
+	    if (NET_CanSendMessage(client->netconnection)) {
+		NET_SendMessage(client->netconnection, &client->message);
+		SZ_Clear(&client->message);
+	    } else {
+		NET_GetMessage(client->netconnection);
+		count++;
 	    }
 	}
 	if ((Sys_DoubleTime() - start) > 3.0)
 	    break;
-    }
-    while (count);
+    } while (count);
 
-// make sure all the clients know we're disconnecting
+    /* make sure all the clients know we're disconnecting */
     buf.data = message;
     buf.maxsize = 4;
     buf.cursize = 0;
     MSG_WriteByte(&buf, svc_disconnect);
     count = NET_SendToAll(&buf, 5);
     if (count)
-	Con_Printf
-	    ("Host_ShutdownServer: NET_SendToAll failed for %u clients\n",
-	     count);
+	Con_Printf("%s: NET_SendToAll failed for %u clients\n", __func__,
+		   count);
 
-    for (i = 0, host_client = svs.clients; i < svs.maxclients;
-	 i++, host_client++)
-	if (host_client->active)
-	    SV_DropClient(crash);
+    client = svs.clients;
+    for (i = 0; i < svs.maxclients; i++, client++)
+	if (client->active)
+	    SV_DropClient(client, crash);
 
-//
-// clear structures
-//
+    /* clear structures */
     memset(&sv, 0, sizeof(sv));
     memset(svs.clients, 0, svs.maxclientslimit * sizeof(client_t));
 }
@@ -559,7 +547,7 @@ Host_GetConsoleCommands(void)
 	cmd = Sys_ConsoleInput();
 	if (!cmd)
 	    break;
-	Cbuf_AddText(cmd);
+	Cbuf_AddText("%s", cmd);
     }
 }
 
@@ -664,7 +652,7 @@ _Host_Frame(float time)
     int pass1, pass2, pass3;
 
     /* something bad happened, or the server disconnected */
-    if (setjmp(host_abortserver))
+    if (setjmp(host_abort))
 	return;
 
     /* keep the random time dependent */
@@ -728,6 +716,7 @@ _Host_Frame(float time)
 	time1 = Sys_DoubleTime();
 
     SCR_UpdateScreen();
+    CL_RunParticles();
 
     if (host_speeds.value)
 	time2 = Sys_DoubleTime();
@@ -751,6 +740,7 @@ _Host_Frame(float time)
     }
 
     host_framecount++;
+    fps_count++;
 }
 
 void
@@ -788,64 +778,6 @@ Host_Frame(float time)
     Con_Printf("serverprofile: %2i clients %2i msec\n", c, m);
 }
 
-//============================================================================
-
-
-#define	VCR_SIGNATURE	0x56435231
-// "VCR1"
-
-void
-Host_InitVCR(quakeparms_t *parms)
-{
-    int i, len, n;
-    char *p;
-
-    if (COM_CheckParm("-playback")) {
-	if (com_argc != 2)
-	    Sys_Error("No other parameters allowed with -playback");
-
-	Sys_FileOpenRead("quake.vcr", &vcrFile);
-	if (vcrFile == -1)
-	    Sys_Error("playback file not found");
-
-	Sys_FileRead(vcrFile, &i, sizeof(int));
-	if (i != VCR_SIGNATURE)
-	    Sys_Error("Invalid signature in vcr file");
-
-	Sys_FileRead(vcrFile, &com_argc, sizeof(int));
-	com_argv = malloc(com_argc * sizeof(char *));
-	com_argv[0] = parms->argv[0];
-	for (i = 0; i < com_argc; i++) {
-	    Sys_FileRead(vcrFile, &len, sizeof(int));
-	    p = malloc(len);
-	    Sys_FileRead(vcrFile, p, len);
-	    com_argv[i + 1] = p;
-	}
-	com_argc++;		/* add one for arg[0] */
-	parms->argc = com_argc;
-	parms->argv = com_argv;
-    }
-
-    if ((n = COM_CheckParm("-record")) != 0) {
-	vcrFile = Sys_FileOpenWrite("quake.vcr");
-
-	i = VCR_SIGNATURE;
-	Sys_FileWrite(vcrFile, &i, sizeof(int));
-	i = com_argc - 1;
-	Sys_FileWrite(vcrFile, &i, sizeof(int));
-	for (i = 1; i < com_argc; i++) {
-	    if (i == n) {
-		len = 10;
-		Sys_FileWrite(vcrFile, &len, sizeof(int));
-		Sys_FileWrite(vcrFile, "-playback", len);
-		continue;
-	    }
-	    len = strlen(com_argv[i]) + 1;
-	    Sys_FileWrite(vcrFile, &len, sizeof(int));
-	    Sys_FileWrite(vcrFile, com_argv[i], len);
-	}
-    }
-}
 
 /*
 ====================
@@ -855,7 +787,6 @@ Host_Init
 void
 Host_Init(quakeparms_t *parms)
 {
-
     if (standard_quake)
 	minimum_memory = MINIMUM_MEMORY;
     else
@@ -867,7 +798,7 @@ Host_Init(quakeparms_t *parms)
     host_parms = *parms;
 
     if (parms->memsize < minimum_memory)
-	Sys_Error("Only %4.1f megs of memory available, can't execute game",
+	Sys_Error("Only %4.1f megs of memory reported, can't execute game",
 		  parms->memsize / (float)0x100000);
 
     com_argc = parms->argc;
@@ -878,15 +809,14 @@ Host_Init(quakeparms_t *parms)
     Cmd_Init();
     V_Init();
     Chase_Init();
-    Host_InitVCR(parms);
-    COM_Init(parms->basedir);
+    COM_Init();
     Host_InitLocal();
-    W_LoadWadFile("gfx.wad");
+    W_LoadWadFile(&host_gfx, "gfx.wad");
     Key_Init();
     Con_Init();
     M_Init();
     PR_Init();
-    Mod_Init();
+    Mod_Init(R_ModelLoader());
     NET_Init();
     SV_Init();
 
@@ -896,63 +826,40 @@ Host_Init(quakeparms_t *parms)
     R_InitTextures();		// needed even for dedicated servers
 
     if (cls.state != ca_dedicated) {
-	host_basepal = (byte *)COM_LoadHunkFile("gfx/palette.lmp");
+	host_basepal = COM_LoadHunkFile("gfx/palette.lmp");
 	if (!host_basepal)
 	    Sys_Error("Couldn't load gfx/palette.lmp");
-	host_colormap = (byte *)COM_LoadHunkFile("gfx/colormap.lmp");
+	host_colormap = COM_LoadHunkFile("gfx/colormap.lmp");
 	if (!host_colormap)
 	    Sys_Error("Couldn't load gfx/colormap.lmp");
 
-#if 0
-#ifndef _WIN32
-	/*
-	 * On non win32, mouse comes before video for security reasons
-	 * FIXME - huh?
-	 */
-	IN_Init ();
-#endif
-#endif
-
 	VID_Init(host_basepal);
 
 	Draw_Init();
 	SCR_Init();
 	R_Init();
 
-#ifndef	_WIN32
-	/*
-	 * On Win32, sound initialization has to come before video
-	 * initialization, so we can put up a popup if the sound hardware is
-	 * in use
-	 */
 	S_Init();
-#else
-
-#ifdef	GLQUAKE
-	// FIXME: doesn't use the new one-window approach yet
-	S_Init();
-#endif
-
-#endif // _WIN32
 	CDAudio_Init();
+
 	Sbar_Init();
 	CL_Init();
 
-	//#ifdef _WIN32
-	// on non win32, mouse comes before video for security reasons
 	IN_Init();
-	//#endif
     }
+    Mod_InitAliasCache();
 
     Hunk_AllocName(0, "-HOST_HUNKLEVEL-");
     host_hunklevel = Hunk_LowMark();
 
-    Cbuf_InsertText("exec quake.rc\n");
-    Cbuf_Execute();
-
     host_initialized = true;
-
     Sys_Printf("========Quake Initialized=========\n");
+
+    /* In case exec of quake.rc fails */
+    if (!setjmp(host_abort)) {
+	Cbuf_InsertText("exec quake.rc\n");
+	Cbuf_Execute();
+    }
 }
 
 
