@@ -18,9 +18,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
+/* FIXME - just for the htons() call below */
+#ifdef WIN32
+#include <winsock2.h>
+#else
+#include <arpa/inet.h>
+#endif
+
 #include "cmd.h"
 #include "console.h"
-#include "net_vcr.h"
+#include "net.h"
 #include "quakedef.h"
 #include "server.h"
 #include "sys.h"
@@ -30,25 +37,13 @@ qsocket_t *net_activeSockets = NULL;
 qsocket_t *net_freeSockets = NULL;
 static int net_numsockets = 0;
 
-qboolean serialAvailable = false;
-qboolean ipxAvailable = false;
 qboolean tcpipAvailable = false;
 
 int net_hostport;
 int DEFAULTnet_hostport = 26000;
 
-char my_ipx_address[NET_NAMELEN];
 char my_tcpip_address[NET_NAMELEN];
 
-void (*GetComPortConfig)(int portNumber, int *port, int *irq,
-			 int *baud, qboolean *useModem);
-void (*SetComPortConfig)(int portNumber, int port, int irq, int baud,
-			 qboolean useModem);
-void (*GetModemConfig)(int portNumber, char *dialType, char *clear,
-		       char *init, char *hangup);
-void (*SetModemConfig)(int portNumber, char *dialType, char *clear,
-		       char *init, char *hangup);
-
 static qboolean listening = false;
 
 qboolean slistInProgress = false;
@@ -75,26 +70,22 @@ int unreliableMessagesReceived = 0;
 cvar_t net_messagetimeout = { "net_messagetimeout", "300" };
 cvar_t hostname = { "hostname", "UNNAMED" };
 
-qboolean configRestored = false;
-cvar_t config_com_port = { "_config_com_port", "0x3f8", true };
-cvar_t config_com_irq = { "_config_com_irq", "4", true };
-cvar_t config_com_baud = { "_config_com_baud", "57600", true };
-cvar_t config_com_modem = { "_config_com_modem", "1", true };
-cvar_t config_modem_dialtype = { "_config_modem_dialtype", "T", true };
-cvar_t config_modem_clear = { "_config_modem_clear", "ATZ", true };
-cvar_t config_modem_init = { "_config_modem_init", "", true };
-cvar_t config_modem_hangup = { "_config_modem_hangup", "AT H", true };
-
-#ifdef IDGODS
-cvar_t idgods = { "idgods", "0" };
-#endif
-
-qboolean recording = false;
-
 net_driver_t *net_driver;
 double net_time;
 
 
+const char *
+NET_AdrToString(const netadr_t *a)
+{
+    static char s[64];
+    const byte *b = a->ip.b;
+
+    sprintf(s, "%i.%i.%i.%i:%i", b[0], b[1], b[2], b[3], ntohs(a->port));
+
+    return s;
+}
+
+
 double
 SetNetTime(void)
 {
@@ -387,7 +378,7 @@ int hostCacheCount = 0;
 hostcache_t hostcache[HOSTCACHESIZE];
 
 qsocket_t *
-NET_Connect(char *host)
+NET_Connect(const char *host)
 {
     qsocket_t *ret;
     int i, n;
@@ -461,17 +452,11 @@ NET_Connect(char *host)
  * NET_CheckNewConnections
  * ===================
  */
-static struct {
-    double time;
-    int op;
-    long session;
-} vcrConnect;
-
 qsocket_t *
 NET_CheckNewConnections(void)
 {
     int i;
-    qsocket_t *ret;
+    qsocket_t *ret = NULL;
 
     SetNetTime();
 
@@ -482,26 +467,11 @@ NET_CheckNewConnections(void)
 	if (!IS_LOOP_DRIVER(net_driver) && listening == false)
 	    continue;
 	ret = net_driver->CheckNewConnections();
-	if (ret) {
-	    if (recording) {
-		vcrConnect.time = host_time;
-		vcrConnect.op = VCR_OP_CONNECT;
-		vcrConnect.session = (long)ret;
-		Sys_FileWrite(vcrFile, &vcrConnect, sizeof(vcrConnect));
-		Sys_FileWrite(vcrFile, ret->address, NET_NAMELEN);
-	    }
-	    return ret;
-	}
+	if (ret)
+	    break;
     }
 
-    if (recording) {
-	vcrConnect.time = host_time;
-	vcrConnect.op = VCR_OP_CONNECT;
-	vcrConnect.session = 0;
-	Sys_FileWrite(vcrFile, &vcrConnect, sizeof(vcrConnect));
-    }
-
-    return NULL;
+    return ret;
 }
 
 /*
@@ -538,14 +508,6 @@ NET_Close(qsocket_t *sock)
  * returns -1 if connection is invalid
  * =================
  */
-static struct {
-    double time;
-    int op;
-    long session;
-    int ret;
-    int len;
-} vcrGetMessage;
-
 int
 NET_GetMessage(qsocket_t *sock)
 {
@@ -571,7 +533,6 @@ NET_GetMessage(qsocket_t *sock)
 	}
     }
 
-
     if (ret > 0) {
 	if (!IS_LOOP_DRIVER(sock->driver)) {
 	    sock->lastMessageTime = net_time;
@@ -580,24 +541,6 @@ NET_GetMessage(qsocket_t *sock)
 	    else if (ret == 2)
 		unreliableMessagesReceived++;
 	}
-
-	if (recording) {
-	    vcrGetMessage.time = host_time;
-	    vcrGetMessage.op = VCR_OP_GETMESSAGE;
-	    vcrGetMessage.session = (long)sock;
-	    vcrGetMessage.ret = ret;
-	    vcrGetMessage.len = net_message.cursize;
-	    Sys_FileWrite(vcrFile, &vcrGetMessage, 24);
-	    Sys_FileWrite(vcrFile, net_message.data, net_message.cursize);
-	}
-    } else {
-	if (recording) {
-	    vcrGetMessage.time = host_time;
-	    vcrGetMessage.op = VCR_OP_GETMESSAGE;
-	    vcrGetMessage.session = (long)sock;
-	    vcrGetMessage.ret = ret;
-	    Sys_FileWrite(vcrFile, &vcrGetMessage, 20);
-	}
     }
 
     return ret;
@@ -615,15 +558,8 @@ NET_GetMessage(qsocket_t *sock)
  * returns -1 : if the connection died
  * ==================
  */
-static struct {
-    double time;
-    int op;
-    long session;
-    int r;
-} vcrSendMessage;
-
 int
-NET_SendMessage(qsocket_t *sock, sizebuf_t *data)
+NET_SendMessage(qsocket_t *sock, const sizebuf_t *data)
 {
     int r;
 
@@ -635,26 +571,17 @@ NET_SendMessage(qsocket_t *sock, sizebuf_t *data)
 	return -1;
     }
 
-
     SetNetTime();
     r = sock->driver->QSendMessage(sock, data);
     if (r == 1 && !IS_LOOP_DRIVER(sock->driver))
 	messagesSent++;
 
-    if (recording) {
-	vcrSendMessage.time = host_time;
-	vcrSendMessage.op = VCR_OP_SENDMESSAGE;
-	vcrSendMessage.session = (long)sock;
-	vcrSendMessage.r = r;
-	Sys_FileWrite(vcrFile, &vcrSendMessage, 20);
-    }
-
     return r;
 }
 
 
 int
-NET_SendUnreliableMessage(qsocket_t *sock, sizebuf_t *data)
+NET_SendUnreliableMessage(qsocket_t *sock, const sizebuf_t *data)
 {
     int r;
 
@@ -671,14 +598,6 @@ NET_SendUnreliableMessage(qsocket_t *sock, sizebuf_t *data)
     if (r == 1 && !IS_LOOP_DRIVER(sock->driver))
 	unreliableMessagesSent++;
 
-    if (recording) {
-	vcrSendMessage.time = host_time;
-	vcrSendMessage.op = VCR_OP_SENDMESSAGE;
-	vcrSendMessage.session = (long)sock;
-	vcrSendMessage.r = r;
-	Sys_FileWrite(vcrFile, &vcrSendMessage, 20);
-    }
-
     return r;
 }
 
@@ -706,68 +625,60 @@ NET_CanSendMessage(qsocket_t *sock)
 
     r = sock->driver->CanSendMessage(sock);
 
-    if (recording) {
-	vcrSendMessage.time = host_time;
-	vcrSendMessage.op = VCR_OP_CANSENDMESSAGE;
-	vcrSendMessage.session = (long)sock;
-	vcrSendMessage.r = r;
-	Sys_FileWrite(vcrFile, &vcrSendMessage, 20);
-    }
-
     return r;
 }
 
 
 int
-NET_SendToAll(sizebuf_t *data, int blocktime)
+NET_SendToAll(const sizebuf_t *data, double blocktime)
 {
+    client_t *client;
     double start;
-    int i;
-    int count = 0;
-    qboolean state1[MAX_SCOREBOARD];
-    qboolean state2[MAX_SCOREBOARD];
+    int i, count;
+    qboolean msg_init[MAX_SCOREBOARD]; /* data written */
+    qboolean msg_sent[MAX_SCOREBOARD]; /* send completed */
 
-    for (i = 0, host_client = svs.clients; i < svs.maxclients;
-	 i++, host_client++) {
-	if (!host_client->netconnection)
+    count = 0;
+    client = svs.clients;
+    for (i = 0; i < svs.maxclients; i++, client++) {
+	if (!client->netconnection || !client->active) {
+	    msg_init[i] = true;
+	    msg_sent[i] = true;
 	    continue;
-	if (host_client->active) {
-	    if (host_client->netconnection->driver == net_drivers) {
-		NET_SendMessage(host_client->netconnection, data);
-		state1[i] = true;
-		state2[i] = true;
-		continue;
-	    }
-	    count++;
-	    state1[i] = false;
-	    state2[i] = false;
-	} else {
-	    state1[i] = true;
-	    state2[i] = true;
 	}
+	/* Loopback driver guarantees delivery, skip checks */
+	if (IS_LOOP_DRIVER(client->netconnection->driver)) {
+	    NET_SendMessage(client->netconnection, data);
+	    msg_init[i] = true;
+	    msg_sent[i] = true;
+	    continue;
+	}
+	count++;
+	msg_init[i] = false;
+	msg_sent[i] = false;
     }
 
     start = Sys_DoubleTime();
     while (count) {
 	count = 0;
-	for (i = 0, host_client = svs.clients; i < svs.maxclients;
-	     i++, host_client++) {
-	    if (!state1[i]) {
-		if (NET_CanSendMessage(host_client->netconnection)) {
-		    state1[i] = true;
-		    NET_SendMessage(host_client->netconnection, data);
+	client = svs.clients;
+	for (i = 0; i < svs.maxclients; i++, client++) {
+	    if (!msg_init[i]) {
+		if (NET_CanSendMessage(client->netconnection)) {
+		    NET_SendMessage(client->netconnection, data);
+		    msg_init[i] = true;
 		} else {
-		    NET_GetMessage(host_client->netconnection);
+		    NET_GetMessage(client->netconnection);
 		}
 		count++;
 		continue;
 	    }
 
-	    if (!state2[i]) {
-		if (NET_CanSendMessage(host_client->netconnection)) {
-		    state2[i] = true;
+	    if (!msg_sent[i]) {
+		if (NET_CanSendMessage(client->netconnection)) {
+		    msg_sent[i] = true;
 		} else {
-		    NET_GetMessage(host_client->netconnection);
+		    NET_GetMessage(client->netconnection);
 		}
 		count++;
 		continue;
@@ -776,6 +687,7 @@ NET_SendToAll(sizebuf_t *data, int blocktime)
 	if ((Sys_DoubleTime() - start) > blocktime)
 	    break;
     }
+
     return count;
 }
 
@@ -788,23 +700,13 @@ NET_SendToAll(sizebuf_t *data, int blocktime)
 void
 NET_Init(void)
 {
-    int i;
+    int i, num_inited;
     int controlSocket;
     qsocket_t *s;
 
-    if (COM_CheckParm("-playback")) {
-	net_numdrivers = 1;
-	net_drivers[0].Init = VCR_Init;
-    }
-
-    if (COM_CheckParm("-record"))
-	recording = true;
-
     i = COM_CheckParm("-port");
     if (!i)
 	i = COM_CheckParm("-udpport");
-    if (!i)
-	i = COM_CheckParm("-ipxport");
 
     if (i) {
 	if (i < com_argc - 1)
@@ -834,17 +736,6 @@ NET_Init(void)
 
     Cvar_RegisterVariable(&net_messagetimeout);
     Cvar_RegisterVariable(&hostname);
-    Cvar_RegisterVariable(&config_com_port);
-    Cvar_RegisterVariable(&config_com_irq);
-    Cvar_RegisterVariable(&config_com_baud);
-    Cvar_RegisterVariable(&config_com_modem);
-    Cvar_RegisterVariable(&config_modem_dialtype);
-    Cvar_RegisterVariable(&config_modem_clear);
-    Cvar_RegisterVariable(&config_modem_init);
-    Cvar_RegisterVariable(&config_modem_hangup);
-#ifdef IDGODS
-    Cvar_RegisterVariable(&idgods);
-#endif
 
     Cmd_AddCommand("slist", NET_Slist_f);
     Cmd_AddCommand("listen", NET_Listen_f);
@@ -852,19 +743,22 @@ NET_Init(void)
     Cmd_AddCommand("port", NET_Port_f);
 
     /* initialize all the drivers */
+    num_inited = 0;
     for (i = 0; i < net_numdrivers; i++) {
 	net_driver = &net_drivers[i];
 	controlSocket = net_driver->Init();
 	if (controlSocket == -1)
 	    continue;
+	num_inited++;
 	net_driver->initialized = true;
 	net_driver->controlSock = controlSocket;
 	if (listening)
 	    net_driver->Listen(true);
     }
 
-    if (*my_ipx_address)
-	Con_DPrintf("IPX address %s\n", my_ipx_address);
+    if (num_inited == 0 && cls.state == ca_dedicated)
+	Sys_Error("Network not available!");
+
     if (*my_tcpip_address)
 	Con_DPrintf("TCP/IP address %s\n", my_tcpip_address);
 }
@@ -895,11 +789,6 @@ NET_Shutdown(void)
 	    net_driver->initialized = false;
 	}
     }
-
-    if (vcrFile != -1) {
-	Con_Printf("Closing vcrfile.\n");
-	Sys_FileClose(vcrFile);
-    }
 }
 
 
@@ -909,24 +798,6 @@ void
 NET_Poll(void)
 {
     PollProcedure *pp;
-    qboolean useModem;
-
-    if (!configRestored) {
-	if (serialAvailable) {
-	    if (config_com_modem.value == 1.0)
-		useModem = true;
-	    else
-		useModem = false;
-	    SetComPortConfig(0, (int)config_com_port.value,
-			     (int)config_com_irq.value,
-			     (int)config_com_baud.value, useModem);
-	    SetModemConfig(0, config_modem_dialtype.string,
-			   config_modem_clear.string,
-			   config_modem_init.string,
-			   config_modem_hangup.string);
-	}
-	configRestored = true;
-    }
 
     SetNetTime();
 
@@ -964,22 +835,3 @@ SchedulePollProcedure(PollProcedure *proc, double timeOffset)
     proc->next = pp;
     prev->next = proc;
 }
-
-
-#ifdef IDGODS
-#define IDNET	0xc0f62800
-
-qboolean
-IsID(struct qsockaddr *addr)
-{
-    if (idgods.value == 0.0)
-	return false;
-
-    if (addr->sa_family != 2)
-	return false;
-
-    if ((BigLong(*(int *)&addr->sa_data[2]) & 0xffffff00) == IDNET)
-	return true;
-    return false;
-}
-#endif
