@@ -23,49 +23,54 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "cmd.h"
 #include "common.h"
 #include "draw.h"
-#include "menu.h"
 #include "protocol.h"
 #include "quakedef.h"
 #include "sbar.h"
 #include "screen.h"
 #include "server.h"
+#include "vid.h"
 #include "wad.h"
 
-int sb_updates;			// if >= vid.numpages, no update needed
+/* if >= vid.numpages, no update needed */
+int sb_updates;
 
-#define STAT_MINUS		10	// num frame for '-' stats digit
-qpic_t *sb_nums[2][11];
-qpic_t *sb_colon, *sb_slash;
-qpic_t *sb_ibar;
-qpic_t *sb_sbar;
-qpic_t *sb_scorebar;
+/* num frame for '-' stats digit */
+#define STAT_MINUS 10
+const qpic8_t *sb_nums[2][11];
+const qpic8_t *sb_colon, *sb_slash;
+const qpic8_t *sb_ibar;
+const qpic8_t *sb_sbar;
+const qpic8_t *sb_scorebar;
 
-qpic_t *sb_weapons[7][8];	// 0 is active, 1 is owned, 2-5 are flashes
-qpic_t *sb_ammo[4];
-qpic_t *sb_sigil[4];
-qpic_t *sb_armor[3];
-qpic_t *sb_items[32];
+/* 0 is active, 1 is owned, 2-5 are flashes */
+const qpic8_t *sb_weapons[7][8];
 
-qpic_t *sb_faces[7][2];		// 0 is gibbed, 1 is dead, 2-6 are alive
+const qpic8_t *sb_ammo[4];
+const qpic8_t *sb_sigil[4];
+const qpic8_t *sb_armor[3];
+const qpic8_t *sb_items[32];
 
-							// 0 is static, 1 is temporary animation
-qpic_t *sb_face_invis;
-qpic_t *sb_face_quad;
-qpic_t *sb_face_invuln;
-qpic_t *sb_face_invis_invuln;
+/* [0][] is gibbed, [1][] is dead, [2-6][] are alive */
+/* [][0] is static, [][1] is temporary animation */
+const qpic8_t *sb_faces[7][2];
+
+const qpic8_t *sb_face_invis;
+const qpic8_t *sb_face_quad;
+const qpic8_t *sb_face_invuln;
+const qpic8_t *sb_face_invis_invuln;
 
 qboolean sb_showscores;
 
 int sb_lines;			// scan lines to draw
 
-qpic_t *rsb_invbar[2];
-qpic_t *rsb_weapons[5];
-qpic_t *rsb_items[2];
-qpic_t *rsb_ammo[3];
-qpic_t *rsb_teambord;		// PGM 01/19/97 - team color border
+const qpic8_t *rsb_invbar[2];
+const qpic8_t *rsb_weapons[5];
+const qpic8_t *rsb_items[2];
+const qpic8_t *rsb_ammo[3];
+const qpic8_t *rsb_teambord;	// PGM 01/19/97 - team color border
 
 //MED 01/04/97 added two more weapons + 3 alternates for grenade launcher
-qpic_t *hsb_weapons[7][5];	// 0 is active, 1 is owned, 2-5 are flashes
+const qpic8_t *hsb_weapons[7][5]; // 0 is active, 1 is owned, 2-5 are flashes
 
 //MED 01/04/97 added array to simplify weapon parsing
 int hipweapons[4] = {
@@ -76,7 +81,7 @@ int hipweapons[4] = {
 };
 
 //MED 01/04/97 added hipnotic items array
-qpic_t *hsb_items[2];
+const qpic8_t *hsb_items[2];
 
 void Sbar_MiniDeathmatchOverlay(void);
 void Sbar_DeathmatchOverlay(void);
@@ -277,7 +282,7 @@ Sbar_DrawPic
 =============
 */
 void
-Sbar_DrawPic(int x, int y, qpic_t *pic)
+Sbar_DrawPic(int x, int y, const qpic8_t *pic)
 {
     if (cl.gametype == GAME_DEATHMATCH)
 	Draw_Pic(x /* + ((vid.width - 320)>>1) */ ,
@@ -293,7 +298,7 @@ Sbar_DrawTransPic
 =============
 */
 void
-Sbar_DrawTransPic(int x, int y, qpic_t *pic)
+Sbar_DrawTransPic(int x, int y, const qpic8_t *pic)
 {
     if (cl.gametype == GAME_DEATHMATCH)
 	Draw_TransPic(x /*+ ((vid.width - 320)>>1) */ ,
@@ -326,8 +331,8 @@ Sbar_DrawCharacter(int x, int y, int num)
 Sbar_DrawString
 ================
 */
-void
-Sbar_DrawString(int x, int y, char *str)
+static void
+Sbar_DrawString(int x, int y, const char *str)
 {
     if (cl.gametype == GAME_DEATHMATCH)
 	Draw_String(x /*+ ((vid.width - 320)>>1) */ ,
@@ -425,7 +430,7 @@ Sbar_SortFrags(void)
 // sort by frags
     scoreboardlines = 0;
     for (i = 0; i < cl.maxclients; i++) {
-	if (cl.scores[i].name[0]) {
+	if (cl.players[i].name[0]) {
 	    fragsort[scoreboardlines] = i;
 	    scoreboardlines++;
 	}
@@ -433,18 +438,21 @@ Sbar_SortFrags(void)
 
     for (i = 0; i < scoreboardlines; i++)
 	for (j = 0; j < scoreboardlines - 1 - i; j++)
-	    if (cl.scores[fragsort[j]].frags <
-		cl.scores[fragsort[j + 1]].frags) {
+	    if (cl.players[fragsort[j]].frags <
+		cl.players[fragsort[j + 1]].frags) {
 		k = fragsort[j];
 		fragsort[j] = fragsort[j + 1];
 		fragsort[j + 1] = k;
 	    }
 }
 
-int
+static int
 Sbar_ColorForMap(int m)
 {
-    return m < 128 ? m + 8 : m + 8;
+    m = qclamp(m, 0, 13) * 16;
+
+    //return m < 128 ? m + 8 : m + 8;
+    return m + 8;
 }
 
 /*
@@ -456,8 +464,7 @@ void
 Sbar_UpdateScoreboard(void)
 {
     int i, k;
-    int top, bottom;
-    scoreboard_t *s;
+    player_info_t *p;
 
     Sbar_SortFrags();
 
@@ -466,13 +473,11 @@ Sbar_UpdateScoreboard(void)
 
     for (i = 0; i < scoreboardlines; i++) {
 	k = fragsort[i];
-	s = &cl.scores[k];
-	sprintf(&scoreboardtext[i][1], "%3i %s", s->frags, s->name);
+	p = &cl.players[k];
+	sprintf(&scoreboardtext[i][1], "%3i %s", p->frags, p->name);
 
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	scoreboardtop[i] = Sbar_ColorForMap(top);
-	scoreboardbottom[i] = Sbar_ColorForMap(bottom);
+	scoreboardtop[i] = Sbar_ColorForMap(p->topcolor);
+	scoreboardbottom[i] = Sbar_ColorForMap(p->bottomcolor);
     }
 }
 
@@ -527,7 +532,7 @@ Sbar_DrawScoreboard(void)
     int x, y;
     int l;
     int top, bottom;
-    scoreboard_t *s;
+    player_info_t *p;
 
     if (cl.gametype != GAME_DEATHMATCH) {
 	Sbar_SoloScoreboard();
@@ -542,15 +547,13 @@ Sbar_DrawScoreboard(void)
 	x = 20 * (i & 1);
 	y = i / 2 * 8;
 
-	s = &cl.scores[fragsort[i]];
-	if (!s->name[0])
+	p = &cl.players[fragsort[i]];
+	if (!p->name[0])
 	    continue;
 
 	// draw background
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
+	top = Sbar_ColorForMap(p->topcolor);
+	bottom = Sbar_ColorForMap(p->bottomcolor);
 
 	Draw_Fill(x * 8 + 10 + ((vid.width - 320) >> 1),
 		  y + vid.height - SBAR_HEIGHT, 28, 4, top);
@@ -594,9 +597,9 @@ Sbar_DrawInventory(void)
 
 // weapons
     for (i = 0; i < 7; i++) {
-	if (cl.items & (IT_SHOTGUN << i)) {
+	if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
 	    time = cl.item_gettime[i];
-	    flashon = (int)((cl.time - time) * 10);
+	    flashon = qmax((int)((cl.time - time) * 10), 0);
 	    if (flashon >= 10) {
 		if (cl.stats[STAT_ACTIVEWEAPON] == (IT_SHOTGUN << i))
 		    flashon = 1;
@@ -618,7 +621,7 @@ Sbar_DrawInventory(void)
 	int grenadeflashing = 0;
 
 	for (i = 0; i < 4; i++) {
-	    if (cl.items & (1 << hipweapons[i])) {
+	    if (cl.stats[STAT_ITEMS] & (1 << hipweapons[i])) {
 		time = cl.item_gettime[hipweapons[i]];
 		flashon = (int)((cl.time - time) * 10);
 		if (flashon >= 10) {
@@ -631,14 +634,14 @@ Sbar_DrawInventory(void)
 
 		// check grenade launcher
 		if (i == 2) {
-		    if (cl.items & HIT_PROXIMITY_GUN) {
+		    if (cl.stats[STAT_ITEMS] & HIT_PROXIMITY_GUN) {
 			if (flashon) {
 			    grenadeflashing = 1;
 			    Sbar_DrawPic(96, -16, hsb_weapons[flashon][2]);
 			}
 		    }
 		} else if (i == 3) {
-		    if (cl.items & (IT_SHOTGUN << 4)) {
+		    if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << 4)) {
 			if (flashon && !grenadeflashing) {
 			    Sbar_DrawPic(96, -16, hsb_weapons[flashon][3]);
 			} else if (!grenadeflashing) {
@@ -679,7 +682,7 @@ Sbar_DrawInventory(void)
     flashon = 0;
     // items
     for (i = 0; i < 6; i++)
-	if (cl.items & (1 << (17 + i))) {
+	if (cl.stats[STAT_ITEMS] & (1 << (17 + i))) {
 	    time = cl.item_gettime[17 + i];
 	    if (time && time > cl.time - 2 && flashon) {	// flash frame
 		sb_updates = 0;
@@ -696,7 +699,7 @@ Sbar_DrawInventory(void)
     // hipnotic items
     if (hipnotic) {
 	for (i = 0; i < 2; i++)
-	    if (cl.items & (1 << (24 + i))) {
+	    if (cl.stats[STAT_ITEMS] & (1 << (24 + i))) {
 		time = cl.item_gettime[24 + i];
 		if (time && time > cl.time - 2 && flashon) {	// flash frame
 		    sb_updates = 0;
@@ -711,7 +714,7 @@ Sbar_DrawInventory(void)
     if (rogue) {
 	// new rogue items
 	for (i = 0; i < 2; i++) {
-	    if (cl.items & (1 << (29 + i))) {
+	    if (cl.stats[STAT_ITEMS] & (1 << (29 + i))) {
 		time = cl.item_gettime[29 + i];
 
 		if (time && time > cl.time - 2 && flashon) {	// flash frame
@@ -727,7 +730,7 @@ Sbar_DrawInventory(void)
     } else {
 	// sigils
 	for (i = 0; i < 4; i++) {
-	    if (cl.items & (1 << (28 + i))) {
+	    if (cl.stats[STAT_ITEMS] & (1 << (28 + i))) {
 		time = cl.item_gettime[28 + i];
 		if (time && time > cl.time - 2 && flashon) {	// flash frame
 		    sb_updates = 0;
@@ -755,7 +758,7 @@ Sbar_DrawFrags(void)
     int x, y, f;
     int xofs;
     char num[12];
-    scoreboard_t *s;
+    player_info_t *p;
 
     Sbar_SortFrags();
 
@@ -771,21 +774,19 @@ Sbar_DrawFrags(void)
 
     for (i = 0; i < l; i++) {
 	k = fragsort[i];
-	s = &cl.scores[k];
-	if (!s->name[0])
+	p = &cl.players[k];
+	if (!p->name[0])
 	    continue;
 
 	// draw background
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
+	top = Sbar_ColorForMap(p->topcolor);
+	bottom = Sbar_ColorForMap(p->bottomcolor);
 
 	Draw_Fill(xofs + x * 8 + 10, y, 28, 4, top);
 	Draw_Fill(xofs + x * 8 + 10, y + 4, 28, 3, bottom);
 
 	// draw number
-	f = s->frags;
+	f = p->frags;
 	sprintf(num, "%3i", f);
 
 	Sbar_DrawCharacter((x + 1) * 8, -24, num[0]);
@@ -821,14 +822,12 @@ Sbar_DrawFace(void)
 	int top, bottom;
 	int xofs;
 	char num[12];
-	scoreboard_t *s;
+	player_info_t *p;
 
-	s = &cl.scores[cl.viewentity - 1];
+	p = &cl.players[cl.viewentity - 1];
 	// draw background
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
+	top = Sbar_ColorForMap(p->topcolor);
+	bottom = Sbar_ColorForMap(p->bottomcolor);
 
 	if (cl.gametype == GAME_DEATHMATCH)
 	    xofs = 113;
@@ -840,7 +839,7 @@ Sbar_DrawFace(void)
 	Draw_Fill(xofs, vid.height - SBAR_HEIGHT + 12, 22, 9, bottom);
 
 	// draw number
-	f = s->frags;
+	f = p->frags;
 	sprintf(num, "%3i", f);
 
 	if (top == 8) {
@@ -860,20 +859,20 @@ Sbar_DrawFace(void)
     }
 // PGM 01/19/97 - team color drawing
 
-    if ((cl.items & (IT_INVISIBILITY | IT_INVULNERABILITY))
+    if ((cl.stats[STAT_ITEMS] & (IT_INVISIBILITY | IT_INVULNERABILITY))
 	== (IT_INVISIBILITY | IT_INVULNERABILITY)) {
 	Sbar_DrawPic(112, 0, sb_face_invis_invuln);
 	return;
     }
-    if (cl.items & IT_QUAD) {
+    if (cl.stats[STAT_ITEMS] & IT_QUAD) {
 	Sbar_DrawPic(112, 0, sb_face_quad);
 	return;
     }
-    if (cl.items & IT_INVISIBILITY) {
+    if (cl.stats[STAT_ITEMS] & IT_INVISIBILITY) {
 	Sbar_DrawPic(112, 0, sb_face_invis);
 	return;
     }
-    if (cl.items & IT_INVULNERABILITY) {
+    if (cl.stats[STAT_ITEMS] & IT_INVULNERABILITY) {
 	Sbar_DrawPic(112, 0, sb_face_invuln);
 	return;
     }
@@ -928,33 +927,33 @@ Sbar_Draw(void)
 	// keys (hipnotic only)
 	//MED 01/04/97 moved keys here so they would not be overwritten
 	if (hipnotic) {
-	    if (cl.items & IT_KEY1)
+	    if (cl.stats[STAT_ITEMS] & IT_KEY1)
 		Sbar_DrawPic(209, 3, sb_items[0]);
-	    if (cl.items & IT_KEY2)
+	    if (cl.stats[STAT_ITEMS] & IT_KEY2)
 		Sbar_DrawPic(209, 12, sb_items[1]);
 	}
 	// armor
-	if (cl.items & IT_INVULNERABILITY) {
+	if (cl.stats[STAT_ITEMS] & IT_INVULNERABILITY) {
 	    Sbar_DrawNum(24, 0, 666, 3, 1);
 	    Sbar_DrawPic(0, 0, draw_disc);
 	} else {
 	    if (rogue) {
 		Sbar_DrawNum(24, 0, cl.stats[STAT_ARMOR], 3,
 			     cl.stats[STAT_ARMOR] <= 25);
-		if (cl.items & RIT_ARMOR3)
+		if (cl.stats[STAT_ITEMS] & RIT_ARMOR3)
 		    Sbar_DrawPic(0, 0, sb_armor[2]);
-		else if (cl.items & RIT_ARMOR2)
+		else if (cl.stats[STAT_ITEMS] & RIT_ARMOR2)
 		    Sbar_DrawPic(0, 0, sb_armor[1]);
-		else if (cl.items & RIT_ARMOR1)
+		else if (cl.stats[STAT_ITEMS] & RIT_ARMOR1)
 		    Sbar_DrawPic(0, 0, sb_armor[0]);
 	    } else {
 		Sbar_DrawNum(24, 0, cl.stats[STAT_ARMOR], 3,
 			     cl.stats[STAT_ARMOR] <= 25);
-		if (cl.items & IT_ARMOR3)
+		if (cl.stats[STAT_ITEMS] & IT_ARMOR3)
 		    Sbar_DrawPic(0, 0, sb_armor[2]);
-		else if (cl.items & IT_ARMOR2)
+		else if (cl.stats[STAT_ITEMS] & IT_ARMOR2)
 		    Sbar_DrawPic(0, 0, sb_armor[1]);
-		else if (cl.items & IT_ARMOR1)
+		else if (cl.stats[STAT_ITEMS] & IT_ARMOR1)
 		    Sbar_DrawPic(0, 0, sb_armor[0]);
 	    }
 	}
@@ -968,28 +967,28 @@ Sbar_Draw(void)
 
 	// ammo icon
 	if (rogue) {
-	    if (cl.items & RIT_SHELLS)
+	    if (cl.stats[STAT_ITEMS] & RIT_SHELLS)
 		Sbar_DrawPic(224, 0, sb_ammo[0]);
-	    else if (cl.items & RIT_NAILS)
+	    else if (cl.stats[STAT_ITEMS] & RIT_NAILS)
 		Sbar_DrawPic(224, 0, sb_ammo[1]);
-	    else if (cl.items & RIT_ROCKETS)
+	    else if (cl.stats[STAT_ITEMS] & RIT_ROCKETS)
 		Sbar_DrawPic(224, 0, sb_ammo[2]);
-	    else if (cl.items & RIT_CELLS)
+	    else if (cl.stats[STAT_ITEMS] & RIT_CELLS)
 		Sbar_DrawPic(224, 0, sb_ammo[3]);
-	    else if (cl.items & RIT_LAVA_NAILS)
+	    else if (cl.stats[STAT_ITEMS] & RIT_LAVA_NAILS)
 		Sbar_DrawPic(224, 0, rsb_ammo[0]);
-	    else if (cl.items & RIT_PLASMA_AMMO)
+	    else if (cl.stats[STAT_ITEMS] & RIT_PLASMA_AMMO)
 		Sbar_DrawPic(224, 0, rsb_ammo[1]);
-	    else if (cl.items & RIT_MULTI_ROCKETS)
+	    else if (cl.stats[STAT_ITEMS] & RIT_MULTI_ROCKETS)
 		Sbar_DrawPic(224, 0, rsb_ammo[2]);
 	} else {
-	    if (cl.items & IT_SHELLS)
+	    if (cl.stats[STAT_ITEMS] & IT_SHELLS)
 		Sbar_DrawPic(224, 0, sb_ammo[0]);
-	    else if (cl.items & IT_NAILS)
+	    else if (cl.stats[STAT_ITEMS] & IT_NAILS)
 		Sbar_DrawPic(224, 0, sb_ammo[1]);
-	    else if (cl.items & IT_ROCKETS)
+	    else if (cl.stats[STAT_ITEMS] & IT_ROCKETS)
 		Sbar_DrawPic(224, 0, sb_ammo[2]);
-	    else if (cl.items & IT_CELLS)
+	    else if (cl.stats[STAT_ITEMS] & IT_CELLS)
 		Sbar_DrawPic(224, 0, sb_ammo[3]);
 	}
 
@@ -1046,18 +1045,18 @@ Sbar_DeathmatchOverlay
 void
 Sbar_DeathmatchOverlay(void)
 {
-    qpic_t *pic;
+    const qpic8_t *pic;
     int i, k, l;
     int top, bottom;
     int x, y, f;
     char num[12];
-    scoreboard_t *s;
+    player_info_t *p;
 
     scr_copyeverything = 1;
     scr_fullupdate = 0;
 
     pic = Draw_CachePic("gfx/ranking.lmp");
-    M_DrawPic((320 - pic->width) / 2, 8, pic);
+    Draw_Pic(160 - pic->width / 2, 8, pic);
 
 // scores
     Sbar_SortFrags();
@@ -1069,21 +1068,19 @@ Sbar_DeathmatchOverlay(void)
     y = 40;
     for (i = 0; i < l; i++) {
 	k = fragsort[i];
-	s = &cl.scores[k];
-	if (!s->name[0])
+	p = &cl.players[k];
+	if (!p->name[0])
 	    continue;
 
 	// draw background
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
+	top = Sbar_ColorForMap(p->topcolor);
+	bottom = Sbar_ColorForMap(p->bottomcolor);
 
 	Draw_Fill(x, y, 40, 4, top);
 	Draw_Fill(x, y + 4, 40, 4, bottom);
 
 	// draw number
-	f = s->frags;
+	f = p->frags;
 	sprintf(num, "%3i", f);
 
 	Draw_Character(x + 8, y, num[0]);
@@ -1099,7 +1096,7 @@ Sbar_DeathmatchOverlay(void)
 	    int n, minutes, tens, units;
 
 	    // draw time
-	    total = cl.completed_time - s->entertime;
+	    total = cl.completed_time - p->entertime;
 	    minutes = (int)total / 60;
 	    n = total - minutes * 60;
 	    tens = n / 10;
@@ -1112,7 +1109,7 @@ Sbar_DeathmatchOverlay(void)
 #endif
 
 	// draw name
-	Draw_String(x + 64, y, s->name);
+	Draw_String(x + 64, y, p->name);
 
 	y += 10;
     }
@@ -1127,92 +1124,61 @@ Sbar_DeathmatchOverlay
 void
 Sbar_MiniDeathmatchOverlay(void)
 {
-    int i, k, l;
-    int top, bottom;
-    int x, y, f;
-    char num[12];
-    scoreboard_t *s;
-    int numlines;
+    int x, y, line, numlines, top, bottom;
 
+    /* Don't bother if not enough room */
     if (vid.width < 512 || !sb_lines)
 	return;
 
     scr_copyeverything = 1;
     scr_fullupdate = 0;
 
-// scores
     Sbar_SortFrags();
 
-// draw the text
-    l = scoreboardlines;
+    /* Check for space to draw the text */
     y = vid.height - sb_lines;
     numlines = sb_lines / 8;
     if (numlines < 3)
 	return;
 
-    //find us
-    for (i = 0; i < scoreboardlines; i++)
-	if (fragsort[i] == cl.viewentity - 1)
+    /* Find client in the scoreboard, if not there (spectator) display top */
+    for (line = 0; line < scoreboardlines; line++)
+	if (fragsort[line] == cl.viewentity - 1)
 	    break;
+    if (line == scoreboardlines)
+	line = 0;
 
-    if (i == scoreboardlines)	// we're not there
-	i = 0;
-    else			// figure out start
-	i = i - numlines / 2;
-
-    if (i > scoreboardlines - numlines)
-	i = scoreboardlines - numlines;
-    if (i < 0)
-	i = 0;
+    /* Put the client in the centre of the displayed lines */
+    line = qclamp(line - numlines / 2, 0, scoreboardlines - numlines);
 
     x = 324;
-    for ( /* */ ; i < scoreboardlines && y < vid.height - 8; i++) {
-	k = fragsort[i];
-	s = &cl.scores[k];
-	if (!s->name[0])
+    while (line < scoreboardlines && y < vid.height - 8 + 1) {
+	const int playernum = fragsort[line++];
+	const player_info_t *player = &cl.players[playernum];
+	if (!player->name[0])
 	    continue;
 
-	// draw background
-	top = s->colors & 0xf0;
-	bottom = (s->colors & 15) << 4;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
-
+	/* draw background */
+	top = Sbar_ColorForMap(player->topcolor);
+	bottom = Sbar_ColorForMap(player->bottomcolor);
 	Draw_Fill(x, y + 1, 40, 3, top);
 	Draw_Fill(x, y + 4, 40, 4, bottom);
 
-	// draw number
-	f = s->frags;
-	sprintf(num, "%3i", f);
-
-	Draw_Character(x + 8, y, num[0]);
-	Draw_Character(x + 16, y, num[1]);
-	Draw_Character(x + 24, y, num[2]);
-
-	if (k == cl.viewentity - 1) {
+	/* draw frags */
+	char frags[4];
+	snprintf(frags, sizeof(frags), "%3d", player->frags);
+	Draw_Character(x + 8, y, frags[0]);
+	Draw_Character(x + 16, y, frags[1]);
+	Draw_Character(x + 24, y, frags[2]);
+	if (playernum == cl.viewentity - 1) {
 	    Draw_Character(x, y, 16);
 	    Draw_Character(x + 32, y, 17);
 	}
-#if 0
-	{
-	    int total;
-	    int n, minutes, tens, units;
 
-	    // draw time
-	    total = cl.completed_time - s->entertime;
-	    minutes = (int)total / 60;
-	    n = total - minutes * 60;
-	    tens = n / 10;
-	    units = n % 10;
-
-	    sprintf(num, "%3i:%i%i", minutes, tens, units);
-
-	    Draw_String(x + 48, y, num);
-	}
-#endif
-
-	// draw name
-	Draw_String(x + 48, y, s->name);
+	/* draw name */
+	char name[17];
+	snprintf(name, sizeof(name), "%-16s", player->name);
+	Draw_String(x + 48, y, name);
 
 	y += 8;
     }
@@ -1227,7 +1193,7 @@ Sbar_IntermissionOverlay
 void
 Sbar_IntermissionOverlay(void)
 {
-    qpic_t *pic;
+    const qpic8_t *pic;
     int dig;
     int num;
 
@@ -1273,7 +1239,7 @@ Sbar_FinaleOverlay
 void
 Sbar_FinaleOverlay(void)
 {
-    qpic_t *pic;
+    const qpic8_t *pic;
 
     scr_copyeverything = 1;
 
