@@ -36,29 +36,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "qtypes.h"
 #include "cvar.h"
 
-#ifndef max
-#if 0
-#define max(a,b) ((a) > (b) ? (a) : (b))
-#define min(a,b) ((a) < (b) ? (a) : (b))
-#else
-
-// Minimal type checking...
-#define max(a,b) ({       \
-    typeof(a) a_ = (a);   \
-    typeof(b) b_ = (b);   \
-    (void)(&a_ == &b_);   \
-    (a_ > b_) ? a_ : b_;  \
-})
-#define min(a,b) ({       \
-    typeof(a) a_ = (a);   \
-    typeof(b) b_ = (b);   \
-    (void)(&a_ == &b_);   \
-    (a_ < b_) ? a_ : b_;  \
-})
-
-#endif
-#endif
-
 //=============================================================================
 
 // the host system specifies the base of the directory tree, the
@@ -66,10 +43,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 // available for the program to use
 
 typedef struct {
-    char *basedir;
-    char *cachedir;		// for development over ISDN lines
+    const char *basedir;
     int argc;
-    char **argv;
+    const char **argv;
     void *membase;
     int memsize;
 } quakeparms_t;
@@ -79,12 +55,8 @@ typedef struct {
 
 #define	MINIMUM_MEMORY	0x550000
 
-#define MAX_NUM_ARGVS	50
-
-
 extern qboolean noclip_anglehack;
 
-
 //
 // host
 //
@@ -102,19 +74,19 @@ extern byte *host_basepal;
 extern byte *host_colormap;
 extern int host_framecount;	// incremented every frame, never reset
 extern double realtime;		// not bounded in any way, changed at
-
-										// start of every frame, never reset
+				// start of every frame, never reset
 
 void Host_ServerFrame(void);
 void Host_InitCommands(void);
 void Host_Init(quakeparms_t *parms);
 void Host_Shutdown(void);
-void Host_Error(char *error, ...);
-void Host_EndGame(char *message, ...);
+void Host_Error(const char *error, ...)
+    __attribute__((noreturn, format(printf,1,2)));
+void Host_EndGame(const char *message, ...)
+    __attribute__((noreturn, format(printf,1,2)));
 qboolean Host_SimulationTime(float time);
 void Host_Frame(float time);
 void Host_Quit_f(void);
-void Host_ClientCommands(char *fmt, ...);
 void Host_ShutdownServer(qboolean crash);
 
 extern qboolean msg_suppress_1;	// suppresses resolution and cache size console
