@@ -38,66 +38,59 @@ netadr_t net_local_adr;
 
 netadr_t net_from;
 sizebuf_t net_message;
-int net_socket;			/* non blocking, for receives */
-int net_send_socket;		/* blocking, for sends */
+int net_socket;
 
 #define	MAX_UDP_PACKET	8192
-byte net_message_buffer[MAX_UDP_PACKET];
+static byte net_message_buffer[MAX_UDP_PACKET];
 
 
-void
+static void
 NetadrToSockadr(netadr_t *a, struct sockaddr_in *s)
 {
     memset(s, 0, sizeof(*s));
     s->sin_family = AF_INET;
 
-    *(int *)&s->sin_addr = *(int *)&a->ip;
+    s->sin_addr.s_addr = a->ip.l;
     s->sin_port = a->port;
 }
 
-void
+static void
 SockadrToNetadr(struct sockaddr_in *s, netadr_t *a)
 {
-    *(int *)&a->ip = *(int *)&s->sin_addr;
+    a->ip.l = s->sin_addr.s_addr;
     a->port = s->sin_port;
 }
 
 qboolean
 NET_CompareBaseAdr(netadr_t a, netadr_t b)
 {
-    if (a.ip[0] == b.ip[0] && a.ip[1] == b.ip[1] && a.ip[2] == b.ip[2]
-	&& a.ip[3] == b.ip[3])
-	return true;
-    return false;
+    return a.ip.l == b.ip.l;
 }
 
 
 qboolean
 NET_CompareAdr(netadr_t a, netadr_t b)
 {
-    if (a.ip[0] == b.ip[0] && a.ip[1] == b.ip[1] && a.ip[2] == b.ip[2]
-	&& a.ip[3] == b.ip[3] && a.port == b.port)
-	return true;
-    return false;
+    return a.ip.l == b.ip.l && a.port == b.port;
 }
 
-char *
+const char *
 NET_AdrToString(netadr_t a)
 {
     static char s[64];
+    const byte *b = a.ip.b;
 
-    sprintf(s, "%i.%i.%i.%i:%i", a.ip[0], a.ip[1], a.ip[2], a.ip[3],
-	    ntohs(a.port));
+    sprintf(s, "%i.%i.%i.%i:%i", b[0], b[1], b[2], b[3], ntohs(a.port));
 
     return s;
 }
 
-char *
+const char *
 NET_BaseAdrToString(netadr_t a)
 {
     static char s[64];
 
-    sprintf(s, "%i.%i.%i.%i", a.ip[0], a.ip[1], a.ip[2], a.ip[3]);
+    sprintf(s, "%i.%i.%i.%i", a.ip.b[0], a.ip.b[1], a.ip.b[2], a.ip.b[3]);
 
     return s;
 }
@@ -113,7 +106,7 @@ NET_BaseAdrToString(netadr_t a)
  * =============
  */
 qboolean
-NET_StringToAdr(char *s, netadr_t *a)
+NET_StringToAdr(const char *s, netadr_t *a)
 {
     struct hostent *h;
     struct sockaddr_in sadr;
@@ -134,11 +127,12 @@ NET_StringToAdr(char *s, netadr_t *a)
 	}
 
     if (copy[0] >= '0' && copy[0] <= '9') {
-	*(int *)&sadr.sin_addr = inet_addr(copy);
+	sadr.sin_addr.s_addr = inet_addr(copy);
     } else {
-	if (!(h = gethostbyname(copy)))
+	h = gethostbyname(copy);
+	if (!h)
 	    return 0;
-	*(int *)&sadr.sin_addr = *(int *)h->h_addr_list[0];
+	sadr.sin_addr.s_addr = *(in_addr_t *)h->h_addr_list[0];
     }
 
     SockadrToNetadr(&sadr, a);
@@ -146,39 +140,6 @@ NET_StringToAdr(char *s, netadr_t *a)
     return true;
 }
 
-/*
- * Returns true if we can't bind the address locally--in other words,
- * the IP is NOT one of our interfaces.
- */
-qboolean
-NET_IsClientLegal(netadr_t *adr)
-{
-#if 0
-    struct sockaddr_in sadr;
-    int newsocket;
-
-    if (adr->ip[0] == 127)
-	return false;		/* no local connections period */
-
-    NetadrToSockadr(adr, &sadr);
-
-    if ((newsocket = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
-	Sys_Error("%s: socket:", __func__, strerror(errno));
-
-    sadr.sin_port = 0;
-
-    if (bind(newsocket, (void *)&sadr, sizeof(sadr)) == -1) {
-	// It is not a local address
-	close(newsocket);
-	return true;
-    }
-    close(newsocket);
-    return false;
-#else
-    return true;
-#endif
-}
-
 
 qboolean
 NET_GetPacket(void)
@@ -196,7 +157,7 @@ NET_GetPacket(void)
 	    return false;
 	if (errno == ECONNREFUSED)
 	    return false;
-	Sys_Printf("NET_GetPacket: %s\n", strerror(errno));
+	Sys_Printf("%s: %s\n", __func__, strerror(errno));
 	return false;
     }
 
@@ -215,35 +176,35 @@ NET_SendPacket(int length, void *data, netadr_t to)
 
     NetadrToSockadr(&to, &addr);
 
-    ret =
-	sendto(net_socket, data, length, 0, (struct sockaddr *)&addr,
-	       sizeof(addr));
+    ret = sendto(net_socket, data, length, 0, (struct sockaddr *)&addr,
+		 sizeof(addr));
     if (ret == -1) {
 	if (errno == EWOULDBLOCK)
 	    return;
 	if (errno == ECONNREFUSED)
 	    return;
-	Sys_Printf("NET_SendPacket: %s\n", strerror(errno));
+	Sys_Printf("%s: %s\n", __func__, strerror(errno));
     }
 }
 
 
-int
+static int
 UDP_OpenSocket(int port)
 {
     int newsocket;
     struct sockaddr_in address;
-    qboolean _true = true;
+    int _true = 1;
     int i;
 
     if ((newsocket = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
-	Sys_Error("%s: socket:", __func__, strerror(errno));
-    if (ioctl(newsocket, FIONBIO, (char *)&_true) == -1)
-	Sys_Error("%s: ioctl FIONBIO:", __func__, strerror(errno));
+	Sys_Error("%s: socket: %s", __func__, strerror(errno));
+    if (ioctl(newsocket, FIONBIO, &_true) == -1)
+	Sys_Error("%s: ioctl FIONBIO: %s", __func__, strerror(errno));
     address.sin_family = AF_INET;
 
     /* ZOID -- check for interface binding option */
-    if ((i = COM_CheckParm("-ip")) != 0 && i < com_argc) {
+    i = COM_CheckParm("-ip");
+    if (i && i < com_argc - 1) {
 	address.sin_addr.s_addr = inet_addr(com_argv[i + 1]);
 	Con_Printf("Binding to IP Interface Address of %s\n",
 		   inet_ntoa(address.sin_addr));
@@ -253,7 +214,7 @@ UDP_OpenSocket(int port)
 	address.sin_port = 0;
     else
 	address.sin_port = htons((short)port);
-    if (bind(newsocket, (void *)&address, sizeof(address)) == -1)
+    if (bind(newsocket, (struct sockaddr *)&address, sizeof(address)) == -1)
 	Sys_Error("%s: bind: %s", __func__, strerror(errno));
 
     return newsocket;
@@ -273,7 +234,7 @@ NET_GetLocalAddress(void)
 
     namelen = sizeof(address);
     if (getsockname(net_socket, (struct sockaddr *)&address, &namelen) == -1)
-	Sys_Error("%s: getsockname:", __func__, strerror(errno));
+	Sys_Error("%s: getsockname: %s", __func__, strerror(errno));
     net_local_adr.port = address.sin_port;
 
     Con_Printf("IP address %s\n", NET_AdrToString(net_local_adr));
