@@ -19,6 +19,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // console.c
 
+#include <string.h>
+
 #include "client.h"
 #include "cmd.h"
 #include "console.h"
@@ -204,38 +206,6 @@ Con_CheckResize(void)
     Con_Resize(&con_main);
 }
 
-
-/*
-================
-Con_Init
-================
-*/
-void
-Con_Init(void)
-{
-    debuglog = COM_CheckParm("-condebug");
-
-    con_main.text = Hunk_AllocName(CON_TEXTSIZE, "conmain");
-
-    con = &con_main;
-    con_linewidth = -1;
-    Con_CheckResize();
-
-    Con_Printf("Console initialized.\n");
-
-//
-// register our commands
-//
-    Cvar_RegisterVariable(&con_notifytime);
-
-    Cmd_AddCommand("toggleconsole", Con_ToggleConsole_f);
-    Cmd_AddCommand("messagemode", Con_MessageMode_f);
-    Cmd_AddCommand("messagemode2", Con_MessageMode2_f);
-    Cmd_AddCommand("clear", Con_Clear_f);
-    con_initialized = true;
-}
-
-
 /*
 ===============
 Con_Linefeed
@@ -262,7 +232,7 @@ If no console is visible, the notify window will pop up.
 ================
 */
 void
-Con_Print(char *txt)
+Con_Print(const char *txt)
 {
     int y;
     int c, l;
@@ -322,7 +292,6 @@ Con_Print(char *txt)
 		con->x = 0;
 	    break;
 	}
-
     }
 }
 
@@ -334,17 +303,15 @@ Con_Printf
 Handles cursor positioning, line wrapping, etc
 ================
 */
-#define	MAXPRINTMSG	4096
-// FIXME: make a buffer size safe vsprintf?
 void
-Con_Printf(char *fmt, ...)
+Con_Printf(const char *fmt, ...)
 {
     va_list argptr;
-    char msg[MAXPRINTMSG];
+    char msg[MAX_PRINTMSG];
     static qboolean inupdate;
 
     va_start(argptr, fmt);
-    vsprintf(msg, fmt, argptr);
+    vsnprintf(msg, sizeof(msg), fmt, argptr);
     va_end(argptr);
 
 // also echo to debugging console
@@ -397,16 +364,24 @@ A Con_Printf that only shows up if the "developer" cvar is set
 ================
 */
 void
-Con_DPrintf(char *fmt, ...)
+Con_DPrintf(const char *fmt, ...)
 {
     va_list argptr;
-    char msg[MAXPRINTMSG];
+    char msg[MAX_PRINTMSG];
 
-    if (!developer.value)
-	return;			// don't confuse non-developers with techie stuff...
+    if (!developer.value) {
+	if (debuglog) {
+	    strcpy(msg, "DEBUG: ");
+	    va_start(argptr, fmt);
+	    vsnprintf(msg + 7, sizeof(msg) - 7, fmt, argptr);
+	    va_end(argptr);
+	    Sys_DebugLog(va("%s/qconsole.log", com_gamedir), "%s", msg);
+	}
+	return;
+    }
 
     va_start(argptr, fmt);
-    vsprintf(msg, fmt, argptr);
+    vsnprintf(msg, sizeof(msg), fmt, argptr);
     va_end(argptr);
 
     Con_Printf("%s", msg);
@@ -454,9 +429,8 @@ Con_DrawInput(void)
 
 // draw it
     y = con_vislines - 22;
-
     for (i = 0; i < con_linewidth; i++)
-	Draw_Character((i + 1) << 3, con_vislines - 22, text[i]);
+	Draw_Character((i + 1) << 3, y, text[i]);
 
 // remove cursor
     key_lines[edit_line][key_linepos] = 0;
@@ -533,6 +507,62 @@ Con_DrawNotify(void)
 	con_notifylines = v;
 }
 
+static void
+Con_DrawDLBar(void)
+{
+#ifdef QW_HACK
+    char dlbar[256];
+    const char *dlname;
+    char *buf;
+    int bufspace;
+    int namespace, len;
+    int barchars, totalchars, markpos;
+    int i;
+
+    if (!cls.download)
+	return;
+
+    /*
+     * totalchars = the space available for the whole bar + text
+     * barchars   = the space for just the bar
+     *
+     * Bar looks like this:
+     *  filename     <====*====> 42%
+     *  longfilen... <==*======> 31%
+     */
+    dlname = COM_SkipPath(cls.downloadname);
+
+    buf = dlbar;
+    bufspace = sizeof(dlbar) - 1;
+
+    namespace  = qmin(con_linewidth / 3, 20);
+    totalchars = qmin(con_linewidth - 2, (int)sizeof(dlbar) - 1);
+
+    if (strlen(dlname) > namespace) {
+	len = snprintf(buf, bufspace, "%.*s... ", namespace - 2, dlname);
+    } else {
+	len = snprintf(buf, bufspace, "%-*s ", namespace + 1, dlname);
+    }
+    buf += len;
+    totalchars -= len;
+    barchars = totalchars - 7; /* 7 => 2 endcaps, space, "100%" */
+
+    markpos = barchars * cls.downloadpercent / 100;
+    *buf++ = '\x80';
+    for (i = 0; i < barchars; i++) {
+	if (i == markpos)
+	    *buf++ = '\x83';
+	else
+	    *buf++ = '\x81';
+    }
+    *buf++ = '\x82';
+    snprintf(buf, 6, " %3d%%", cls.downloadpercent);
+
+    /* draw it */
+    Draw_String(8, con_vislines - 22 + 8, dlbar);
+#endif
+}
+
 /*
 ================
 Con_DrawConsole
@@ -549,11 +579,6 @@ Con_DrawConsole(int lines)
     char *text;
     int row;
 
-#ifdef QW_HACK
-    int j, n;
-    char dlbar[1024];
-#endif
-
     if (lines <= 0)
 	return;
 
@@ -586,50 +611,8 @@ Con_DrawConsole(int lines)
 	    Draw_Character((x + 1) << 3, y, text[x]);
     }
 
-#ifdef QW_HACK
-    // draw the download bar
-    // figure out width
-    if (cls.download) {
-	if ((text = strrchr(cls.downloadname, '/')) != NULL)
-	    text++;
-	else
-	    text = cls.downloadname;
-
-	x = con_linewidth - ((con_linewidth * 7) / 40);
-	y = x - strlen(text) - 8;
-	i = con_linewidth / 3;
-	if (strlen(text) > i) {
-	    y = x - i - 11;
-	    strncpy(dlbar, text, i);
-	    dlbar[i] = 0;
-	    strcat(dlbar, "...");
-	} else
-	    strcpy(dlbar, text);
-	strcat(dlbar, ": ");
-	i = strlen(dlbar);
-	dlbar[i++] = '\x80';
-	// where's the dot go?
-	if (cls.downloadpercent == 0)
-	    n = 0;
-	else
-	    n = y * cls.downloadpercent / 100;
-
-	for (j = 0; j < y; j++)
-	    if (j == n)
-		dlbar[i++] = '\x83';
-	    else
-		dlbar[i++] = '\x81';
-	dlbar[i++] = '\x82';
-	dlbar[i] = 0;
-
-	sprintf(dlbar + strlen(dlbar), " %02d%%", cls.downloadpercent);
-
-	// draw it
-	y = con_vislines - 22 + 8;
-	for (i = 0; i < strlen(dlbar); i++)
-	    Draw_Character((i + 1) << 3, y, dlbar[i]);
-    }
-#endif
+    // draw the download bar, if needed
+    Con_DrawDLBar();
 
     // draw the input prompt, user text, and cursor if desired
     Con_DrawInput();
@@ -650,7 +633,7 @@ Con_NotifyBox(char *text)
     Con_Printf
 	("\n\n\35\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\37\n");
 
-    Con_Printf(text);
+    Con_Printf("%s", text);
 
     Con_Printf("Press a key.\n");
     Con_Printf
@@ -681,14 +664,14 @@ Okay to call even when the screen can't be updated
 ==================
 */
 void
-Con_SafePrintf(char *fmt, ...)
+Con_SafePrintf(const char *fmt, ...)
 {
     va_list argptr;
-    char msg[1024];
+    char msg[MAX_PRINTMSG];
     int temp;
 
     va_start(argptr, fmt);
-    vsprintf(msg, fmt, argptr);
+    vsnprintf(msg, sizeof(msg), fmt, argptr);
     va_end(argptr);
 
     temp = scr_disabled_for_loading;
@@ -698,28 +681,21 @@ Con_SafePrintf(char *fmt, ...)
 }
 
 void
-Con_ShowList(const char **list, int cnt)
+Con_ShowList(const char **list, int cnt, int maxlen)
 {
     const char *s;
-    unsigned i, j, max_len, len, cols, rows;
+    unsigned i, j, len, cols, rows;
     char *line;
 
     /* Lay them out in columns */
-    max_len = 0;
-    for (i = 0; i < cnt; ++i) {
-	len = strlen(list[i]);
-	if (len > max_len)
-	    max_len = len;
-    }
-
     line = Z_Malloc(Con_GetWidth() + 1);
-    cols = Con_GetWidth() / (max_len + 2);
-    rows = cnt / cols + 1;
+    cols = Con_GetWidth() / (maxlen + 2);
+    rows = cnt / cols + ((cnt % cols) ? 1 : 0);
 
     /* Looks better if we have a few rows before spreading out */
     if (rows < 5) {
-	cols = cnt / 5 + 1;
-	rows = cnt / cols + 1;
+	cols = cnt / 5 + ((cnt % 5) ? 1 : 0);
+	rows = cnt / cols + ((cnt % cols) ? 1 : 0);
     }
 
     for (i = 0; i < rows; ++i) {
@@ -732,7 +708,7 @@ Con_ShowList(const char **list, int cnt)
 
 	    strcat(line, s);
 	    if (j < cols - 1) {
-		while (len < max_len) {
+		while (len < maxlen) {
 		    strcat(line, " ");
 		    len++;
 		}
@@ -743,3 +719,75 @@ Con_ShowList(const char **list, int cnt)
     }
     Z_Free(line);
 }
+
+static const char **showtree_list;
+static unsigned showtree_idx;
+
+static void
+Con_ShowTree_Populate(struct rb_node *n)
+{
+    if (n) {
+	Con_ShowTree_Populate(n->rb_left);
+	showtree_list[showtree_idx++] = stree_entry(n)->string;
+	Con_ShowTree_Populate(n->rb_right);
+    }
+}
+
+void
+Con_ShowTree(struct stree_root *root)
+{
+    /* FIXME - cheating with malloc */
+    showtree_list = malloc(root->entries * sizeof(char *));
+    if (showtree_list) {
+	showtree_idx = 0;
+	Con_ShowTree_Populate(root->root.rb_node);
+	Con_ShowList(showtree_list, root->entries, root->maxlen);
+	free(showtree_list);
+    }
+}
+
+
+void
+Con_Maplist_f()
+{
+    struct stree_root st_root = STREE_ROOT;
+    const char *pfx = NULL;
+
+    if (Cmd_Argc() == 2)
+	pfx = Cmd_Argv(1);
+
+    STree_AllocInit();
+    COM_ScanDir(&st_root, "maps", pfx, ".bsp", true);
+    Con_ShowTree(&st_root);
+}
+
+
+/*
+================
+Con_Init
+================
+*/
+void
+Con_Init(void)
+{
+    debuglog = COM_CheckParm("-condebug");
+
+    con_main.text = Hunk_AllocName(CON_TEXTSIZE, "conmain");
+
+    con = &con_main;
+    con_linewidth = -1;
+    Con_CheckResize();
+
+    Con_Printf("Console initialized.\n");
+
+    /* register our commands */
+    Cvar_RegisterVariable(&con_notifytime);
+
+    Cmd_AddCommand("toggleconsole", Con_ToggleConsole_f);
+    Cmd_AddCommand("messagemode", Con_MessageMode_f);
+    Cmd_AddCommand("messagemode2", Con_MessageMode2_f);
+    Cmd_AddCommand("clear", Con_Clear_f);
+    Cmd_AddCommand("maplist", Con_Maplist_f);
+
+    con_initialized = true;
+}
