@@ -35,7 +35,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifdef SERVERONLY
 #include "qwsvdef.h"
 #include "server.h"
-void SV_SendServerInfoChange(char *key, char *value);	// FIXME
 #else
 #ifdef QW_HACK
 #include "quakedef.h"
@@ -43,8 +42,10 @@ void SV_SendServerInfoChange(char *key, char *value);	// FIXME
 #endif
 #endif
 
-cvar_t *cvar_vars;
-static char *cvar_null_string = "";
+static const char *cvar_null_string = "";
+
+#define cvar_entry(ptr) container_of(ptr, struct cvar_s, stree)
+DECLARE_STREE_ROOT(cvar_tree);
 
 /*
 ============
@@ -52,24 +53,88 @@ Cvar_FindVar
 ============
 */
 cvar_t *
-Cvar_FindVar(char *var_name)
+Cvar_FindVar(const char *var_name)
 {
-    cvar_t *var;
+    struct cvar_s *ret = NULL;
+    struct stree_node *n;
 
-    for (var = cvar_vars; var; var = var->next)
-	if (!strcmp(var_name, var->name))
-	    return var;
+    n = STree_Find(&cvar_tree, var_name);
+    if (n)
+	ret = cvar_entry(n);
 
-    return NULL;
+    return ret;
 }
 
+/*
+ * Return a string tree with all possible argument completions of the given
+ * buffer for the given cvar.
+ */
+struct stree_root *
+Cvar_ArgCompletions(const char *name, const char *buf)
+{
+    cvar_t *cvar;
+    struct stree_root *root = NULL;
+
+    cvar = Cvar_FindVar(name);
+    if (cvar && cvar->completion)
+	root = cvar->completion(buf);
+
+    return root;
+}
+
+/*
+ * Call the argument completion function for cvar "name".
+ * Returned result should be Z_Free'd after use.
+ */
+char *
+Cvar_ArgComplete(const char *name, const char *buf)
+{
+    char *result = NULL;
+    struct stree_root *root;
+
+    root = Cvar_ArgCompletions(name, buf);
+    if (root) {
+	result = STree_MaxMatch(root, buf);
+	Z_Free(root);
+    }
+
+    return result;
+}
+
+
+#ifdef NQ_HACK
+/*
+ * For NQ/net_dgrm.c, command == CCREQ_RULE_INFO case
+ */
+cvar_t *
+Cvar_NextServerVar(const char *var_name)
+{
+    cvar_t *ret = NULL;
+    cvar_t *var;
+    struct stree_node *n;
+
+    if (var_name[0] == '\0')
+	var_name = NULL;
+
+    STree_ForEach_After(&cvar_tree, n, var_name) {
+	var = cvar_entry(n);
+	if (var->server) {
+	    ret = var;
+	    STree_ForEach_break(&cvar_tree);
+	}
+    }
+
+    return ret;
+}
+#endif
+
 /*
 ============
 Cvar_VariableValue
 ============
 */
 float
-Cvar_VariableValue(char *var_name)
+Cvar_VariableValue(const char *var_name)
 {
     cvar_t *var;
 
@@ -85,8 +150,8 @@ Cvar_VariableValue(char *var_name)
 Cvar_VariableString
 ============
 */
-char *
-Cvar_VariableString(char *var_name)
+const char *
+Cvar_VariableString(const char *var_name)
 {
     cvar_t *var;
 
@@ -97,49 +162,21 @@ Cvar_VariableString(char *var_name)
 }
 
 
-/*
-============
-Cvar_CompleteVariable
-============
-*/
-char *
-Cvar_CompleteVariable(char *partial)
-{
-    cvar_t *cvar;
-    int len;
-
-    len = strlen(partial);
-
-    if (!len)
-	return NULL;
-
-    // check exact match
-    for (cvar = cvar_vars; cvar; cvar = cvar->next)
-	if (!strcmp(partial, cvar->name))
-	    return cvar->name;
-
-    // check partial match
-    for (cvar = cvar_vars; cvar; cvar = cvar->next)
-	if (!strncmp(partial, cvar->name, len))
-	    return cvar->name;
-
-    return NULL;
-}
-
-
 /*
 ============
 Cvar_Set
 ============
 */
 void
-Cvar_Set(char *var_name, char *value)
+Cvar_Set(const char *var_name, const char *value)
 {
     cvar_t *var;
+    char *newstring;
     qboolean changed;
 
     var = Cvar_FindVar(var_name);
-    if (!var) {			// there is an error in C code if this happens
+    if (!var) {
+	/* there is an error in C code if this happens */
 	Con_Printf("Cvar_Set: variable %s not found\n", var_name);
 	return;
     }
@@ -149,14 +186,14 @@ Cvar_Set(char *var_name, char *value)
 	return;
     }
 
+    changed = strcmp(var->string, value);
+
     /* Check for developer-only cvar */
-    if ((var->flags & CVAR_DEVELOPER) && !developer.value) {
+    if (changed && (var->flags & CVAR_DEVELOPER) && !developer.value) {
 	Con_Printf("%s is settable only in developer mode.\n", var_name);
 	return;
     }
 
-    changed = strcmp(var->string, value);
-
 #ifdef SERVERONLY
     if (var->info) {
 	Info_SetValueForKey(svs.info, var_name, value, MAX_SERVERINFO_STRING);
@@ -169,8 +206,8 @@ Cvar_Set(char *var_name, char *value)
 	Info_SetValueForKey(cls.userinfo, var_name, value, MAX_INFO_STRING);
 	if (cls.state >= ca_connected) {
 	    MSG_WriteByte(&cls.netchan.message, clc_stringcmd);
-	    SZ_Print(&cls.netchan.message,
-		     va("setinfo \"%s\" \"%s\"\n", var_name, value));
+	    MSG_WriteStringf(&cls.netchan.message, "setinfo \"%s\" \"%s\"\n",
+			     var_name, value);
 	}
     }
 #endif
@@ -178,8 +215,9 @@ Cvar_Set(char *var_name, char *value)
 
     Z_Free(var->string);	// free the old value string
 
-    var->string = Z_Malloc(strlen(value) + 1);
-    strcpy(var->string, value);
+    newstring = Z_Malloc(strlen(value) + 1);
+    strcpy(newstring, value);
+    var->string = newstring;
     var->value = Q_atof(var->string);
 
 #ifdef NQ_HACK
@@ -208,11 +246,11 @@ Cvar_SetValue
 ============
 */
 void
-Cvar_SetValue(char *var_name, float value)
+Cvar_SetValue(const char *var_name, float value)
 {
     char val[32];
 
-    sprintf(val, "%f", value);
+    snprintf(val, sizeof(val), "%f", value);
     Cvar_Set(var_name, val);
 }
 
@@ -230,40 +268,36 @@ Cvar_RegisterVariable(cvar_t *variable)
     char value[512];		// FIXME - magic numbers...
     float old_developer;
 
-// first check to see if it has allready been defined
+    /* first check to see if it has already been defined */
     if (Cvar_FindVar(variable->name)) {
-	Con_Printf("Can't register variable %s, allready defined\n",
+	Con_Printf("Can't register variable %s, already defined\n",
 		   variable->name);
 	return;
     }
-// check for overlap with a command
+    /* check for overlap with a command */
     if (Cmd_Exists(variable->name)) {
 	Con_Printf("Cvar_RegisterVariable: %s is a command\n",
 		   variable->name);
 	return;
     }
-// link the variable in
-    variable->next = cvar_vars;
-    cvar_vars = variable;
+
+    variable->stree.string = variable->name;
+    STree_Insert(&cvar_tree, &variable->stree);
 
 // copy the value off, because future sets will Z_Free it
     strncpy(value, variable->string, 511);
     value[511] = '\0';
     variable->string = Z_Malloc(1);
 
+    /*
+     * FIXME (BARF) - readonly cvars need to be initialised
+     *                developer 1 allows set
+     */
     /* set it through the function to be consistant */
-    /* FIXME (BARF) - readonly cvars need to be initialised; developer 1 allows set */
     old_developer = developer.value;
     developer.value = 1;
     Cvar_Set(variable->name, value);
     developer.value = old_developer;
-
-    /* Add the name to the completion cache */
-    insert_cvar_completion(variable->name);
-
-    /* Insert into own completion cache */
-    //variable->index.string = variable->name;
-    //rb_insert_completion(&cvar_completions, &variable->index);
 }
 
 /*
@@ -309,8 +343,11 @@ void
 Cvar_WriteVariables(FILE *f)
 {
     cvar_t *var;
+    struct stree_node *n;
 
-    for (var = cvar_vars; var; var = var->next)
+    STree_ForEach(&cvar_tree, n) {
+	var = cvar_entry(n);
 	if (var->archive)
 	    fprintf(f, "%s \"%s\"\n", var->name, var->string);
+    }
 }
