@@ -28,20 +28,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "common.h"
 #include "console.h"
 #include "input.h"
+#include "model.h"
 #include "quakedef.h"
 #include "sound.h"
 #include "sys.h"
 
-#ifdef GLQUAKE
-#include "gl_model.h"
-#else
-#include "model.h"
-#endif
-
-#ifdef _WIN32
-#include "winquake.h"
-#endif
-
 #ifdef NQ_HACK
 #include "host.h"
 #endif
@@ -85,7 +76,6 @@ static sfx_t *ambient_sfx[NUM_AMBIENTS];
 
 static int sound_started = 0;
 
-cvar_t bgmvolume = { "bgmvolume", "1", true };
 cvar_t volume = { "volume", "0.7", true };
 cvar_t loadas8bit = { "loadas8bit", "0" };
 
@@ -130,13 +120,14 @@ S_SoundInfo_f(void)
 	return;
     }
 
-    Con_Printf("%5d stereo\n", shm->channels - 1);
+    Con_Printf("%5d channels (%s)\n", shm->channels,
+	       shm->channels == 1 ? "mono" : "stereo");
     Con_Printf("%5d samples\n", shm->samples);
     Con_Printf("%5d samplepos\n", shm->samplepos);
     Con_Printf("%5d samplebits\n", shm->samplebits);
     Con_Printf("%5d submission_chunk\n", shm->submission_chunk);
     Con_Printf("%5d speed\n", shm->speed);
-    Con_Printf("0x%x dma buffer\n", shm->buffer);
+    Con_Printf("%p dma buffer\n", shm->buffer);
     Con_Printf("%5d total_channels\n", total_channels);
 }
 
@@ -156,9 +147,7 @@ S_Startup(void)
     if (!fakedma) {
 	rc = SNDDMA_Init();
 	if (!rc) {
-#ifndef	_WIN32
-	    Con_Printf("S_Startup: SNDDMA_Init failed.\n");
-#endif
+	    Con_Printf("%s: SNDDMA_Init failed.\n", __func__);
 	    sound_started = 0;
 	    return;
 	}
@@ -192,7 +181,6 @@ S_Init(void)
     Cvar_RegisterVariable(&volume);
     Cvar_RegisterVariable(&precache);
     Cvar_RegisterVariable(&loadas8bit);
-    Cvar_RegisterVariable(&bgmvolume);
     Cvar_RegisterVariable(&bgmbuffer);
     Cvar_RegisterVariable(&ambient_level);
     Cvar_RegisterVariable(&ambient_fade);
@@ -217,19 +205,17 @@ S_Init(void)
     /* create a piece of DMA memory */
     if (fakedma) {
 	shm = (void *)Hunk_AllocName(sizeof(*shm), "shm");
-	shm->splitbuffer = 0;
 	shm->samplebits = 16;
 	shm->speed = 22050;
 	shm->channels = 2;
 	shm->samples = 32768;
 	shm->samplepos = 0;
-	shm->soundalive = true;
-	shm->gamealive = true;
 	shm->submission_chunk = 1;
 	shm->buffer = Hunk_AllocName(1 << 16, "shmbuf");
     }
 
-    Con_Printf("Sound sampling rate: %i\n", shm->speed);
+    if (sound_started)
+	Con_Printf("Sound sampling rate: %i\n", shm->speed);
 
 #if 0
     /* provides a tick sound until washed clean; for debugging */
@@ -253,8 +239,6 @@ S_Shutdown(void)
 
     if (!sound_started)
 	return;
-    if (shm)
-	shm->gamealive = 0;
 
     shm = 0;
     sound_started = 0;
@@ -270,7 +254,7 @@ S_Shutdown(void)
  * ==================
  */
 static sfx_t *
-S_FindName(char *name)
+S_FindName(const char *name)
 {
     int i;
     sfx_t *sfx;
@@ -303,7 +287,7 @@ S_FindName(char *name)
  * ==================
  */
 void
-S_TouchSound(char *name)
+S_TouchSound(const char *name)
 {
     sfx_t *sfx;
 
@@ -320,7 +304,7 @@ S_TouchSound(char *name)
  * ==================
  */
 sfx_t *
-S_PrecacheSound(char *name)
+S_PrecacheSound(const char *name)
 {
     sfx_t *sfx;
 
@@ -356,7 +340,7 @@ SND_PickChannel(int entnum, int entchannel)
 	channel = &channels[i];
 	/*
 	 * - channel 0 never overrides
-	 * - allways override sound from same entity
+	 * - always override sound from same entity
 	 */
 	if (entchannel != 0 && channel->entnum == entnum &&
 	    (channel->entchannel == entchannel || entchannel == -1)) {
@@ -397,9 +381,8 @@ SND_Spatialize(channel_t *ch)
     vec_t dist;
     vec_t lscale, rscale, scale;
     vec3_t source_vec;
-    sfx_t *snd;
 
-    /* anything coming from the view entity will allways be full volume */
+    /* anything coming from the view entity will always be full volume */
 #ifdef NQ_HACK
     if (ch->entnum == cl.viewentity) {
 	ch->leftvol = ch->master_vol;
@@ -416,7 +399,6 @@ SND_Spatialize(channel_t *ch)
 #endif
 
     /* calculate stereo seperation and distance attenuation */
-    snd = ch->sfx;
     VectorSubtract(ch->origin, listener_origin, source_vec);
     dist = VectorNormalize(source_vec) * ch->dist_mult;
 
@@ -553,50 +535,21 @@ S_StopAllSoundsC(void)
 void
 S_ClearBuffer(void)
 {
+    int err;
     int clear;
 
-#ifdef _WIN32
-    if (!sound_started || !shm || (!shm->buffer && !pDSBuf))
-#else
-    if (!sound_started || !shm || !shm->buffer)
-#endif
+    if (!sound_started || !shm)
 	return;
 
-    if (shm->samplebits == 8)
-	clear = 0x80;
-    else
-	clear = 0;
-
-#ifdef _WIN32
-    if (pDSBuf) {
-	DWORD dwSize;
-	DWORD *pData;
-	int reps;
-	HRESULT hresult;
-
-	reps = 0;
-
-	while ((hresult =
-		pDSBuf->lpVtbl->Lock(pDSBuf, 0, gSndBufSize, &pData, &dwSize,
-				     NULL, NULL, 0)) != DS_OK) {
-	    if (hresult != DSERR_BUFFERLOST) {
-		Con_Printf("S_ClearBuffer: DS::Lock Sound Buffer Failed\n");
-		S_Shutdown();
-		return;
-	    }
-	    if (++reps > 10000) {
-		Con_Printf("S_ClearBuffer: DS: couldn't restore buffer\n");
-		S_Shutdown();
-		return;
-	    }
-	}
-	memset(pData, clear, shm->samples * shm->samplebits / 8);
-	pDSBuf->lpVtbl->Unlock(pDSBuf, pData, dwSize, NULL, 0);
-    } else
-#endif
-    {
-	memset(shm->buffer, clear, shm->samples * shm->samplebits / 8);
+    err = SNDDMA_LockBuffer();
+    if (err) {
+	S_Shutdown();
+	return;
     }
+
+    clear = (shm->samplebits == 8) ? 0x80 : 0;
+    memset(shm->buffer, clear, shm->samples * shm->samplebits / 8);
+    SNDDMA_UnlockBuffer();
 }
 
 
@@ -648,7 +601,7 @@ S_StaticSound(sfx_t *sfx, vec3_t origin, float vol, float attenuation)
 static void
 S_UpdateAmbientSounds(void)
 {
-    mleaf_t *l;
+    mleaf_t *leaf;
     float vol;
     int ambient_channel;
     channel_t *chan;
@@ -660,8 +613,8 @@ S_UpdateAmbientSounds(void)
     if (!cl.worldmodel)
 	return;
 
-    l = Mod_PointInLeaf(listener_origin, cl.worldmodel);
-    if (!l || !ambient_level.value) {
+    leaf = Mod_PointInLeaf(cl.worldmodel, listener_origin);
+    if (!leaf || !ambient_level.value) {
 	for (ambient_channel = 0; ambient_channel < NUM_AMBIENTS;
 	     ambient_channel++)
 	    channels[ambient_channel].sfx = NULL;
@@ -673,7 +626,7 @@ S_UpdateAmbientSounds(void)
 	chan = &channels[ambient_channel];
 	chan->sfx = ambient_sfx[ambient_channel];
 
-	vol = ambient_level.value * l->ambient_sound_level[ambient_channel];
+	vol = ambient_level.value * leaf->ambient_sound_level[ambient_channel];
 	if (vol < 8)
 	    vol = 0;
 
@@ -835,7 +788,8 @@ S_Update_(void)
 
     /* check to make sure that we haven't overshot */
     if (paintedtime < soundtime) {
-	/* Con_Printf ("S_Update_ : overflow\n"); */
+	/* FIXME - handle init & wrap properly and report actual overflow */
+	//Con_DPrintf("%s: overflow\n", __func__);
 	paintedtime = soundtime;
     }
     /* mix ahead of current position */
@@ -844,26 +798,7 @@ S_Update_(void)
     if (endtime - soundtime > samps)
 	endtime = soundtime + samps;
 
-#ifdef _WIN32
-    /* if the buffer was lost or stopped, restore it and/or restart it */
-    {
-	DWORD dwStatus;
-
-	if (pDSBuf) {
-	    if (pDSBuf->lpVtbl->GetStatus(pDSBuf, &dwStatus) != DD_OK)
-		Con_Printf("Couldn't get sound buffer status\n");
-
-	    if (dwStatus & DSBSTATUS_BUFFERLOST)
-		pDSBuf->lpVtbl->Restore(pDSBuf);
-
-	    if (!(dwStatus & DSBSTATUS_PLAYING))
-		pDSBuf->lpVtbl->Play(pDSBuf, 0, 0, DSBPLAY_LOOPING);
-	}
-    }
-#endif
-
     S_PaintChannels(endtime);
-
     SNDDMA_Submit();
 }
 
@@ -945,7 +880,7 @@ S_SoundList(void)
 
 
 void
-S_LocalSound(char *sound)
+S_LocalSound(const char *sound)
 {
     sfx_t *sfx;
 
@@ -956,7 +891,7 @@ S_LocalSound(char *sound)
 
     sfx = S_PrecacheSound(sound);
     if (!sfx) {
-	Con_Printf("S_LocalSound: can't cache %s\n", sound);
+	Con_Printf("%s: can't cache %s\n", __func__, sound);
 	return;
     }
 #ifdef NQ_HACK
