@@ -21,6 +21,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "cmd.h"
 #include "common.h"
 #include "console.h"
+#include "mathlib.h"
 #include "quakedef.h"
 #include "sys.h"
 #include "zone.h"
@@ -45,6 +46,7 @@ typedef struct {
 
 static void Cache_FreeLow(int new_low_hunk);
 static void Cache_FreeHigh(int new_high_hunk);
+static void Cache_Dealloc(cache_user_t *c);
 
 /*
  * ============================================================================
@@ -85,6 +87,7 @@ Z_ClearZone(memzone_t *zone, int size)
     zone->blocklist.id = 0;
     zone->blocklist.size = 0;
     zone->rover = block;
+    zone->size = size;
 
     block->prev = block->next = &zone->blocklist;
     block->tag = 0;		/* free block */
@@ -99,14 +102,14 @@ Z_ClearZone(memzone_t *zone, int size)
  * ========================
  */
 void
-Z_Free(void *ptr)
+Z_Free(const void *ptr)
 {
     memblock_t *block, *other;
 
     if (!ptr)
 	Sys_Error("%s: NULL pointer", __func__);
 
-    block = (memblock_t *) ((byte *)ptr - sizeof(memblock_t));
+    block = (memblock_t *)((const byte *)ptr - sizeof(memblock_t));
     if (block->id != ZONEID)
 	Sys_Error("%s: freed a pointer without ZONEID", __func__);
     if (block->tag == 0)
@@ -143,6 +146,7 @@ Z_Free(void *ptr)
 static void
 Z_CheckHeap(void)
 {
+#ifdef DEBUG
     memblock_t *block;
 
     for (block = mainzone->blocklist.next;; block = block->next) {
@@ -157,13 +161,14 @@ Z_CheckHeap(void)
 	if (!block->tag && !block->next->tag)
 	    Sys_Error("%s: two consecutive free blocks", __func__);
     }
+#endif
 }
 
 
 static void *
 Z_TagMalloc(int size, int tag)
 {
-    int extra;
+    int extra, *marker;
     memblock_t *start, *rover, *new, *base;
 
     if (!tag)
@@ -174,7 +179,7 @@ Z_TagMalloc(int size, int tag)
      * sufficient size
      */
     size += sizeof(memblock_t);	/* account for size of block header */
-    size += 4;			/* space for memory trash tester */
+    size += sizeof(int);	/* space for memory trash tester */
     size = (size + 7) & ~7;	/* align to 8-byte boundary */
 
     base = rover = mainzone->rover;
@@ -195,7 +200,7 @@ Z_TagMalloc(int size, int tag)
     extra = base->size - size;
     if (extra > MINFRAGMENT) {
 	/* there will be a free fragment after the allocated block */
-	new = (memblock_t *) ((byte *)base + size);
+	new = (memblock_t *)((byte *)base + size);
 	new->size = extra;
 	new->tag = 0;		/* free block */
 	new->prev = base;
@@ -212,9 +217,10 @@ Z_TagMalloc(int size, int tag)
     base->id = ZONEID;
 
     /* marker for memory trash testing */
-    *(int *)((byte *)base + base->size - 4) = ZONEID;
+    marker = (int *)((byte *)base + base->size - sizeof(int));
+    *marker = ZONEID;
 
-    return (void *)((byte *)base + sizeof(memblock_t));
+    return base + 1;
 }
 
 
@@ -237,24 +243,68 @@ Z_Malloc(int size)
     return buf;
 }
 
+/*
+ * ========================
+ * Z_Realloc
+ * ========================
+ */
+void *
+Z_Realloc(const void *ptr, int size)
+{
+    memblock_t *block;
+    int orig_size;
+    void *ret;
+
+    if (!ptr)
+	return Z_Malloc(size);
+
+    block = (memblock_t *)((byte *)ptr - sizeof(memblock_t));
+    if (block->id != ZONEID)
+	Sys_Error("%s: realloced a pointer without ZONEID", __func__);
+    if (!block->tag)
+	Sys_Error("%s: realloced a freed pointer", __func__);
+
+    orig_size = block->size;
+    orig_size -= sizeof(memblock_t);
+    orig_size -= sizeof(int); /* ZONEID marker */
+
+    Z_Free(ptr);
+    ret = Z_TagMalloc(size, 1);
+    if (!ret)
+	Sys_Error("%s: failed on allocation of %i bytes", __func__, size);
+    if (ret != ptr)
+	memmove(ret, ptr, qmin(orig_size, size));
+    if (size > orig_size)
+	memset((byte *)ret + orig_size, 0, size - orig_size);
+    return ret;
+}
 
-/* FIXME - unused; Make a console command? */
-#if 0
 /*
  * ========================
  * Z_Print
  * ========================
  */
 static void
-Z_Print(memzone_t * zone)
+Z_Print(const memzone_t *zone, qboolean detailed)
 {
-    memblock_t *block;
+    const memblock_t *block;
+    unsigned free_blocks = 0, used_blocks = 0;
+    size_t free_size = 0, used_size = 0;
 
-    Con_Printf("zone size: %i  location: %p\n", mainzone->size, mainzone);
+    block = zone->blocklist.next;
+    while (block) {
+	if (detailed)
+	    Con_Printf("block:%p    size:%7i    tag:%3i\n",
+		       block, block->size, block->tag);
 
-    for (block = zone->blocklist.next;; block = block->next) {
-	Con_Printf("block:%p    size:%7i    tag:%3i\n",
-		   block, block->size, block->tag);
+	/* Update totals */
+	if (!block->tag) {
+	    free_blocks++;
+	    free_size += block->size;
+	} else {
+	    used_blocks++;
+	    used_size += block->size;
+	}
 
 	if (block->next == &zone->blocklist)
 	    break;		/* all blocks have been hit */
@@ -264,28 +314,51 @@ Z_Print(memzone_t * zone)
 	    Con_Printf("ERROR: next block doesn't have proper back link\n");
 	if (!block->tag && !block->next->tag)
 	    Con_Printf("ERROR: two consecutive free blocks\n");
+
+	block = block->next;
     }
+
+    Con_Printf("zone size: %d  location: %p\n", zone->size, zone);
+    Con_Printf("  %7lu bytes used in %d blocks\n",
+	       (unsigned long)used_size, used_blocks);
+    Con_Printf("  %7lu bytes available in %d blocks\n",
+	       (unsigned long)free_size, free_blocks);
+}
+
+static void
+Z_Zone_f(void)
+{
+    if (Cmd_Argc() == 2) {
+	if (!strcmp(Cmd_Argv(1), "print")) {
+	    Z_Print(mainzone, false);
+	    return;
+	}
+	if (!strcmp(Cmd_Argv(1), "printall")) {
+	    Z_Print(mainzone, true);
+	    return;
+	}
+    }
+    Con_Printf("Usage: zone print\n");
 }
-#endif
 
 /* ======================================================================= */
 
-#define	HUNK_SENTINAL	0x1df001ed
+#define HUNK_SENTINAL 0x1df001ed
 
 typedef struct {
     int sentinal;
-    int size;		/* including sizeof(hunk_t), -1 = not allocated */
-    char name[8];
+    int size; /* including sizeof(hunk_t) */
+    char name[HUNK_NAMELEN]; /* not necessarily NULL terminated */
 } hunk_t;
 
-static byte *hunk_base;
-static int hunk_size;
-
-static int hunk_low_used;
-static int hunk_high_used;
-
-static qboolean hunk_tempactive;
-static int hunk_tempmark;
+static struct {
+    byte *base;
+    void *lowbase;
+    int size;
+    int lowbytes;
+    int highbytes;
+    int tempmark;
+} hunkstate;
 
 /*
  * ==============
@@ -297,20 +370,20 @@ static int hunk_tempmark;
 void
 Hunk_Check(void)
 {
-    hunk_t *h;
+    hunk_t *hunk;
 
-    for (h = (hunk_t *) hunk_base; (byte *)h != hunk_base + hunk_low_used;) {
-	if (h->sentinal != HUNK_SENTINAL)
+    hunk = (hunk_t *)hunkstate.base;
+    while ((byte *)hunk != hunkstate.base + hunkstate.lowbytes) {
+	if (hunk->sentinal != HUNK_SENTINAL)
 	    Sys_Error("%s: trashed sentinal", __func__);
-	if (h->size < 16 || h->size + (byte *)h - hunk_base > hunk_size)
+	if (hunk->size < sizeof(hunk_t) ||
+	    hunk->size + (byte *)hunk - hunkstate.base > hunkstate.size)
 	    Sys_Error("%s: bad size", __func__);
-	h = (hunk_t *) ((byte *)h + h->size);
+	hunk = (hunk_t *)((byte *)hunk + hunk->size);
     }
 }
 
 
-/* FIXME - unused; Make a console command? */
-#if 0
 /*
  * ==============
  * Hunk_Print
@@ -323,81 +396,106 @@ Hunk_Check(void)
 static void
 Hunk_Print(qboolean all)
 {
-    hunk_t *h, *next, *endlow, *starthigh, *endhigh;
-    int count, sum;
+    const hunk_t *hunk, *next, *endlow, *starthigh, *endhigh;
+    int count, sum, pwidth;
     int totalblocks;
-    char name[9];
+    char safename[HUNK_NAMELEN + 1];
 
-    name[8] = 0;
     count = 0;
     sum = 0;
     totalblocks = 0;
+    /*
+     * We use a copy of the hunk name with an extra zero terminator
+     * for printing, in case of hunk corruption.
+     */
+    memset(safename, 0, sizeof(safename));
 
-    h = (hunk_t *) hunk_base;
-    endlow = (hunk_t *) (hunk_base + hunk_low_used);
-    starthigh = (hunk_t *) (hunk_base + hunk_size - hunk_high_used);
-    endhigh = (hunk_t *) (hunk_base + hunk_size);
+    /* Don't put in wide spaces if not printing pointers */
+    pwidth = all ? (sizeof(void *) * 2 + 2) : 8;
 
-    Con_Printf("          :%8i total hunk size\n", hunk_size);
+    endlow = (hunk_t *)(hunkstate.base + hunkstate.lowbytes);
+    endhigh = (hunk_t *)(hunkstate.base + hunkstate.size);
+    starthigh = (hunk_t *)((byte *)endhigh - hunkstate.highbytes);
+
+    Con_Printf("%*s :%10i total hunk size\n", pwidth, "", hunkstate.size);
     Con_Printf("-------------------------\n");
 
+    next = (hunk_t *)hunkstate.base;
     while (1) {
+	hunk = next;
+
 	/*
 	 * skip to the high hunk if done with low hunk
 	 */
-	if (h == endlow) {
-	    Con_Printf("-------------------------\n");
-	    Con_Printf("          :%8i REMAINING\n",
-		       hunk_size - hunk_low_used - hunk_high_used);
-	    Con_Printf("-------------------------\n");
-	    h = starthigh;
+	if (hunk == endlow) {
+	    int mem = hunkstate.size - hunkstate.lowbytes - hunkstate.highbytes;
+	    Con_Printf("-------------------------\n"
+		       "%*s :%10i REMAINING\n"
+		       "-------------------------\n",
+		       pwidth, "", mem);
+	    hunk = starthigh;
 	}
+
 	/*
 	 * if totally done, break
 	 */
-	if (h == endhigh)
+	if (hunk == endhigh)
 	    break;
 
 	/*
 	 * run consistancy checks
 	 */
-	if (h->sentinal != HUNK_SENTINAL)
-	    Sys_Error("%s: trahsed sentinal", __func__);
-	if (h->size < 16 || h->size + (byte *)h - hunk_base > hunk_size)
+	if (hunk->sentinal != HUNK_SENTINAL)
+	    Sys_Error("%s: trashed sentinal", __func__);
+	if (hunk->size < 16)
+	    Sys_Error("%s: bad size", __func__);
+	if (hunk->size + (byte *)hunk - hunkstate.base > hunkstate.size)
 	    Sys_Error("%s: bad size", __func__);
-
-	next = (hunk_t *) ((byte *)h + h->size);
-	count++;
-	totalblocks++;
-	sum += h->size;
 
 	/*
 	 * print the single block
 	 */
-	memcpy(name, h->name, 8);
+	memcpy(safename, hunk->name, HUNK_NAMELEN);
 	if (all)
-	    Con_Printf("%8p :%8i %8s\n", h, h->size, name);
+	    Con_Printf("%*p :%10i %-*s\n", pwidth, hunk, hunk->size,
+		       HUNK_NAMELEN, safename);
 
+	next = (hunk_t *)((byte *)hunk + hunk->size);
+	count++;
+	totalblocks++;
+	sum += hunk->size;
+
+	if (next != endlow && next != endhigh)
+	    if (!strncmp(hunk->name, next->name, HUNK_NAMELEN))
+		continue;
 	/*
 	 * print the total
 	 */
-	if (next == endlow || next == endhigh ||
-	    strncmp(h->name, next->name, 8)) {
-	    if (!all)
-		Con_Printf("          :%8i %8s (TOTAL)\n", sum, name);
-	    count = 0;
-	    sum = 0;
-	}
-
-	h = next;
+	if (!all)
+	    Con_Printf("%*s :%10i %-*s (TOTAL)\n",
+		       pwidth, "", sum, HUNK_NAMELEN, safename);
+	count = 0;
+	sum = 0;
     }
-
     Con_Printf("-------------------------\n");
     Con_Printf("%8i total blocks\n", totalblocks);
-
 }
-#endif
 
+static void
+Hunk_f(void)
+{
+    if (Cmd_Argc() == 2) {
+	if (!strcmp(Cmd_Argv(1), "print")) {
+	    Hunk_Print(false);
+	    return;
+	}
+	if (!strcmp(Cmd_Argv(1), "printall")) {
+	    Hunk_Print(true);
+	    return;
+	}
+    }
+    Con_Printf("Usage: hunk print|printall\n");
+}
 
 /*
  * ===================
@@ -405,9 +503,10 @@ Hunk_Print(qboolean all)
  * ===================
  */
 void *
-Hunk_AllocName(int size, char *name)
+Hunk_AllocName(int size, const char *name)
 {
-    hunk_t *h;
+    hunk_t *hunk;
+    size_t freebytes;
 
 #ifdef PARANOID
     Hunk_Check();
@@ -418,7 +517,62 @@ Hunk_AllocName(int size, char *name)
 
     size = sizeof(hunk_t) + ((size + 15) & ~15);
 
-    if (hunk_size - hunk_low_used - hunk_high_used < size) {
+    freebytes = hunkstate.size - hunkstate.lowbytes - hunkstate.highbytes;
+    if (freebytes < size) {
+	int extra, newmem;
+
+	/* check how much was needed, but recommend at least 1/4 extra */
+	extra = (size - freebytes + (1 << 20) - 1) >> 20;
+	extra = qmax(extra, hunkstate.size >> 22);
+
+	/* Round memory recommendation to a multiple of 16MB */
+	newmem = (((hunkstate.size + (1 << 20) - 1) >> 20) + extra + 15) & ~15;
+
+	Sys_Error ("%s: failed on %d bytes.\n\n"
+		   "Not enough RAM allocated (%4.1f MB).\n"
+		   "Try starting using \"-mem %d\" on the command line.",
+		   __func__, size, (float)hunkstate.size / 1024 / 1024, newmem);
+    }
+
+    hunk = (hunk_t *)(hunkstate.base + hunkstate.lowbytes);
+    hunkstate.lowbytes += size;
+
+    Cache_FreeLow(hunkstate.lowbytes);
+
+    memset(hunk, 0, size);
+
+    hunk->size = size;
+    hunk->sentinal = HUNK_SENTINAL;
+    memset(hunk->name, 0, HUNK_NAMELEN);
+    memcpy(hunk->name, name, qmin((int)strlen(name), HUNK_NAMELEN));
+
+    /* Save a copy of the allocated address */
+    hunkstate.lowbase = hunk + 1;
+
+    return hunkstate.lowbase;
+}
+
+/*
+ * Hunk_AllocExtend
+ *
+ * Extend the current hunk allocation, rather than allocating a new
+ * hunk_t header. Useful for sequencing lots of small allocations.
+ */
+void *
+Hunk_AllocExtend(const void *base, int size)
+{
+    hunk_t *hunk;
+    void *memptr;
+
+    if (base != hunkstate.lowbase)
+	Sys_Error("%s: base != current base", __func__);
+
+    if (size < 0)
+	Sys_Error("%s: bad size: %i", __func__, size);
+
+    size = (size + 15) & ~15;
+
+    if (hunkstate.size - hunkstate.lowbytes - hunkstate.highbytes < size) {
 	/* Sys_Error ("%s: failed on %i bytes", __func__, size); */
 #ifdef _WIN32
 	Sys_Error("Not enough RAM allocated.  Try starting using "
@@ -429,18 +583,18 @@ Hunk_AllocName(int size, char *name)
 #endif
     }
 
-    h = (hunk_t *) (hunk_base + hunk_low_used);
-    hunk_low_used += size;
+    hunk = (hunk_t *)base - 1;
+    if (hunk->sentinal != HUNK_SENTINAL)
+	Sys_Error("%s: bad sentinal (%d)", __func__, hunk->sentinal);
 
-    Cache_FreeLow(hunk_low_used);
+    hunkstate.lowbytes += size;
+    Cache_FreeLow(hunkstate.lowbytes);
 
-    memset(h, 0, size);
+    memptr = (byte *)hunk + hunk->size;
+    memset(memptr, 0, size);
+    hunk->size += size;
 
-    h->size = size;
-    h->sentinal = HUNK_SENTINAL;
-    strncpy(h->name, name, 8);
-
-    return (void *)(h + 1);
+    return memptr;
 }
 
 /*
@@ -457,40 +611,46 @@ Hunk_Alloc(int size)
 int
 Hunk_LowMark(void)
 {
-    return hunk_low_used;
+    return hunkstate.lowbytes;
 }
 
 void
 Hunk_FreeToLowMark(int mark)
 {
-    if (mark < 0 || mark > hunk_low_used)
+    if (mark < 0 || mark > hunkstate.lowbytes)
 	Sys_Error("%s: bad mark %i", __func__, mark);
-    memset(hunk_base + mark, 0, hunk_low_used - mark);
-    hunk_low_used = mark;
+    memset(hunkstate.base + mark, 0, hunkstate.lowbytes - mark);
+    hunkstate.lowbytes = mark;
 }
 
 int
 Hunk_HighMark(void)
 {
-    if (hunk_tempactive) {
-	hunk_tempactive = false;
-	Hunk_FreeToHighMark(hunk_tempmark);
+    if (hunkstate.tempmark) {
+	const int tempmark = hunkstate.tempmark;
+	hunkstate.tempmark = 0;
+	Hunk_FreeToHighMark(tempmark);
     }
 
-    return hunk_high_used;
+    return hunkstate.highbytes;
 }
 
 void
 Hunk_FreeToHighMark(int mark)
 {
-    if (hunk_tempactive) {
-	hunk_tempactive = false;
-	Hunk_FreeToHighMark(hunk_tempmark);
+    byte *base;
+
+    if (hunkstate.tempmark) {
+	const int tempmark = hunkstate.tempmark;
+	hunkstate.tempmark = 0;
+	Hunk_FreeToHighMark(tempmark);
     }
-    if (mark < 0 || mark > hunk_high_used)
+    if (mark < 0 || mark > hunkstate.highbytes)
 	Sys_Error("%s: bad mark %i", __func__, mark);
-    memset(hunk_base + hunk_size - hunk_high_used, 0, hunk_high_used - mark);
-    hunk_high_used = mark;
+
+    base = hunkstate.base + hunkstate.size - hunkstate.highbytes;
+    memset(base, 0, hunkstate.highbytes - mark);
+    hunkstate.highbytes = mark;
 }
 
 
@@ -500,16 +660,17 @@ Hunk_FreeToHighMark(int mark)
  * ===================
  */
 void *
-Hunk_HighAllocName(int size, char *name)
+Hunk_HighAllocName(int size, const char *name)
 {
-    hunk_t *h;
+    hunk_t *hunk;
 
     if (size < 0)
 	Sys_Error("%s: bad size: %i", __func__, size);
 
-    if (hunk_tempactive) {
-	Hunk_FreeToHighMark(hunk_tempmark);
-	hunk_tempactive = false;
+    if (hunkstate.tempmark) {
+	const int tempmark = hunkstate.tempmark;
+	hunkstate.tempmark = 0;
+	Hunk_FreeToHighMark(tempmark);
     }
 #ifdef PARANOID
     Hunk_Check();
@@ -517,22 +678,22 @@ Hunk_HighAllocName(int size, char *name)
 
     size = sizeof(hunk_t) + ((size + 15) & ~15);
 
-    if (hunk_size - hunk_low_used - hunk_high_used < size) {
+    if (hunkstate.size - hunkstate.lowbytes - hunkstate.highbytes < size) {
 	Con_Printf("Hunk_HighAlloc: failed on %i bytes\n", size);
 	return NULL;
     }
 
-    hunk_high_used += size;
-    Cache_FreeHigh(hunk_high_used);
+    hunkstate.highbytes += size;
+    Cache_FreeHigh(hunkstate.highbytes);
 
-    h = (hunk_t *) (hunk_base + hunk_size - hunk_high_used);
+    hunk = (hunk_t *)(hunkstate.base + hunkstate.size - hunkstate.highbytes);
 
-    memset(h, 0, size);
-    h->size = size;
-    h->sentinal = HUNK_SENTINAL;
-    strncpy(h->name, name, 8);
+    memset(hunk, 0, size);
+    hunk->size = size;
+    hunk->sentinal = HUNK_SENTINAL;
+    snprintf(hunk->name, sizeof(hunk->name), "%s", name);
 
-    return (void *)(h + 1);
+    return (void *)(hunk + 1);
 }
 
 
@@ -547,23 +708,61 @@ void *
 Hunk_TempAlloc(int size)
 {
     void *buf;
+    int mark;
 
     size = (size + 15) & ~15;
 
-    if (hunk_tempactive) {
-	Hunk_FreeToHighMark(hunk_tempmark);
-	hunk_tempactive = false;
+    if (hunkstate.tempmark) {
+	const int tempmark = hunkstate.tempmark;
+	hunkstate.tempmark = 0;
+	Hunk_FreeToHighMark(tempmark);
     }
 
-    hunk_tempmark = Hunk_HighMark();
-
+    mark = Hunk_HighMark();
     buf = Hunk_HighAllocName(size, "temp");
-
-    hunk_tempactive = true;
+    hunkstate.tempmark = mark;
 
     return buf;
 }
 
+/*
+ * =====================
+ * Hunk_TempAllocExtend
+ *
+ * Extend the existing temp hunk allocation.
+ * Size is the number of extra bytes required
+ * =====================
+ */
+void *
+Hunk_TempAllocExtend(int size)
+{
+    hunk_t *old, *new;
+
+    if (!hunkstate.tempmark)
+	Sys_Error("%s: temp hunk not active", __func__);
+
+    old = (hunk_t *)(hunkstate.base + hunkstate.size - hunkstate.highbytes);
+    if (old->sentinal != HUNK_SENTINAL)
+	Sys_Error("%s: old sentinal trashed\n", __func__);
+    if (strncmp(old->name, "temp", HUNK_NAMELEN))
+	Sys_Error("%s: old hunk name trashed\n", __func__);
+
+    size = (size + 15) & ~15;
+    if (hunkstate.size - hunkstate.lowbytes - hunkstate.highbytes < size) {
+	Con_Printf("%s: failed on %i bytes\n", __func__, size);
+	return NULL;
+    }
+
+    hunkstate.highbytes += size;
+    Cache_FreeHigh(hunkstate.highbytes);
+
+    new = (hunk_t *)(hunkstate.base + hunkstate.size - hunkstate.highbytes);
+    memmove(new, old, sizeof(hunk_t));
+    new->size += size;
+
+    return new + 1;
+}
+
 /*
  * ===========================================================================
  *
@@ -572,10 +771,12 @@ Hunk_TempAlloc(int size)
  * ===========================================================================
  */
 
+#define CACHE_NAMELEN 32
+
 typedef struct cache_system_s {
     int size;			/* including this header */
     cache_user_t *user;
-    char name[16];
+    char name[CACHE_NAMELEN];
     struct cache_system_s *prev, *next;
     struct cache_system_s *lru_prev, *lru_next;	/* for LRU flushing */
 } cache_system_t;
@@ -583,27 +784,63 @@ typedef struct cache_system_s {
 static cache_system_t cache_head;
 static cache_system_t *Cache_TryAlloc(int size, qboolean nobottom);
 
+static inline cache_system_t *
+Cache_System(const cache_user_t *c)
+{
+    return (cache_system_t *)((byte *)c->data - c->pad) - 1;
+}
+
+static inline void *
+Cache_Data(const cache_system_t *c)
+{
+    return (byte *)(c + 1) + c->user->pad;
+}
+
+#ifdef DEBUG
+void
+Cache_CheckLinks(void)
+{
+    const cache_system_t *cache;
+
+    cache = cache_head.next;
+    while (cache != &cache_head)
+	cache = cache->next;
+
+    cache = cache_head.prev;
+    while (cache != &cache_head)
+	cache = cache->prev;
+
+    cache = cache_head.lru_next;
+    while (cache != &cache_head)
+	cache = cache->lru_next;
+
+    cache = cache_head.lru_prev;
+    while (cache != &cache_head)
+	cache = cache->lru_prev;
+}
+#endif
+
 /*
  * ===========
  * Cache_Move
  * ===========
  */
 static void
-Cache_Move(cache_system_t *c)
+Cache_Move(cache_system_t *old_cs)
 {
-    cache_system_t *new;
+    cache_system_t *new_cs;
 
     /* we are clearing up space at the bottom, so only allocate it late */
-    new = Cache_TryAlloc(c->size, true);
-    if (new) {
-	memcpy(new + 1, c + 1, c->size - sizeof(cache_system_t));
-	new->user = c->user;
-	memcpy(new->name, c->name, sizeof(new->name));
-	Cache_Free(c->user);
-	new->user->data = (void *)(new + 1);
+    new_cs = Cache_TryAlloc(old_cs->size, true);
+    if (new_cs) {
+	memcpy(new_cs + 1, old_cs + 1, old_cs->size - sizeof(cache_system_t));
+	new_cs->user = old_cs->user;
+	memcpy(new_cs->name, old_cs->name, sizeof(new_cs->name));
+	Cache_Dealloc(old_cs->user);
+	new_cs->user->data = Cache_Data(new_cs);
     } else {
 	/* tough luck... */
-	Cache_Free(c->user);
+	Cache_Free(old_cs->user);
     }
 }
 
@@ -623,7 +860,7 @@ Cache_FreeLow(int new_low_hunk)
 	c = cache_head.next;
 	if (c == &cache_head)
 	    return;		/* nothing in cache at all */
-	if ((byte *)c >= hunk_base + new_low_hunk)
+	if ((byte *)c >= hunkstate.base + new_low_hunk)
 	    return;		/* there is space to grow the hunk */
 	Cache_Move(c);		/* reclaim the space */
     }
@@ -646,7 +883,7 @@ Cache_FreeHigh(int new_high_hunk)
 	c = cache_head.prev;
 	if (c == &cache_head)
 	    return;		/* nothing in cache at all */
-	if ((byte *)c + c->size <= hunk_base + hunk_size - new_high_hunk)
+	if ((byte *)c + c->size <= hunkstate.base + hunkstate.size - new_high_hunk)
 	    return;		/* there is space to grow the hunk */
 	if (c == prev)
 	    Cache_Free(c->user);	/* didn't move out of the way */
@@ -696,10 +933,10 @@ Cache_TryAlloc(int size, qboolean nobottom)
 
     /* is the cache completely empty? */
     if (!nobottom && cache_head.prev == &cache_head) {
-	if (hunk_size - hunk_high_used - hunk_low_used < size)
-	    Sys_Error("%s: %i is greater then free hunk", __func__, size);
+	if (hunkstate.size - hunkstate.highbytes - hunkstate.lowbytes < size)
+	    Sys_Error("%s: %i is greater than free hunk", __func__, size);
 
-	new = (cache_system_t *) (hunk_base + hunk_low_used);
+	new = (cache_system_t *)(hunkstate.base + hunkstate.lowbytes);
 	memset(new, 0, sizeof(*new));
 	new->size = size;
 
@@ -711,7 +948,7 @@ Cache_TryAlloc(int size, qboolean nobottom)
     }
 
     /* search from the bottom up for space */
-    new = (cache_system_t *) (hunk_base + hunk_low_used);
+    new = (cache_system_t *)(hunkstate.base + hunkstate.lowbytes);
     cs = cache_head.next;
 
     do {
@@ -732,13 +969,13 @@ Cache_TryAlloc(int size, qboolean nobottom)
 	}
 
 	/* continue looking */
-	new = (cache_system_t *) ((byte *)cs + cs->size);
+	new = (cache_system_t *)((byte *)cs + cs->size);
 	cs = cs->next;
 
     } while (cs != &cache_head);
 
     /* try to allocate one at the very end */
-    if (hunk_base + hunk_size - hunk_high_used - (byte *)new >= size) {
+    if (hunkstate.base + hunkstate.size - hunkstate.highbytes - (byte *)new >= size) {
 	memset(new, 0, sizeof(*new));
 	new->size = size;
 
@@ -769,9 +1006,6 @@ Cache_Flush(void)
 	Cache_Free(cache_head.next->user);	/* reclaim the space */
 }
 
-
-/* FIXME - Unused? */
-#if 0
 /*
  * ============
  * Cache_Print
@@ -786,7 +1020,6 @@ Cache_Print(void)
 	Con_Printf("%8i : %s\n", cd->size, cd->name);
     }
 }
-#endif
 
 /*
  * ============
@@ -797,8 +1030,8 @@ void
 Cache_Report(void)
 {
     Con_DPrintf("%4.1f megabyte data cache\n",
-		(hunk_size - hunk_high_used -
-		 hunk_low_used) / (float)(1024 * 1024));
+		(hunkstate.size - hunkstate.highbytes -
+		 hunkstate.lowbytes) / (float)(1024 * 1024));
 }
 
 
@@ -829,31 +1062,48 @@ Cache_Init(void)
 
 /*
  * ==============
- * Cache_Free
+ * Cache_Dealloc
  *
  * Frees the memory and removes it from the LRU list
  * ==============
  */
-void
-Cache_Free(cache_user_t *c)
+static void
+Cache_Dealloc(cache_user_t *c)
 {
     cache_system_t *cs;
 
     if (!c->data)
 	Sys_Error("%s: not allocated", __func__);
 
-    cs = ((cache_system_t *) c->data) - 1;
-
+    cs = Cache_System(c);
     cs->prev->next = cs->next;
     cs->next->prev = cs->prev;
     cs->next = cs->prev = NULL;
 
-    c->data = NULL;
-
     Cache_UnlinkLRU(cs);
 }
 
+/*
+ * ==============
+ * Cache_Free
+ *
+ * Call the destructor before freeing the cache entry
+ * ==============
+ */
+void
+Cache_Free(cache_user_t *c)
+{
+    /* Cleanup the user data */
+    if (c->destructor) {
+	c->destructor(c);
+	c->destructor = NULL;
+    }
 
+    Cache_Dealloc(c);
+
+    c->pad = 0;
+    c->data = NULL;
+}
 
 /*
  * ==============
@@ -861,14 +1111,14 @@ Cache_Free(cache_user_t *c)
  * ==============
  */
 void *
-Cache_Check(cache_user_t *c)
+Cache_Check(const cache_user_t *c)
 {
     cache_system_t *cs;
 
     if (!c->data)
 	return NULL;
 
-    cs = ((cache_system_t *) c->data) - 1;
+    cs = Cache_System(c);
 
     /* move to head of LRU */
     Cache_UnlinkLRU(cs);
@@ -884,28 +1134,41 @@ Cache_Check(cache_user_t *c)
  * ==============
  */
 void *
-Cache_Alloc(cache_user_t *c, int size, char *name)
+Cache_Alloc(cache_user_t *c, int size, const char *name)
+{
+    return Cache_AllocPadded(c, 0, size, name);
+}
+
+/*
+ * ==============
+ * Cache_AllocPadded
+ * ==============
+ */
+void *
+Cache_AllocPadded(cache_user_t *c, int pad, int size, const char *name)
 {
     cache_system_t *cs;
 
     if (c->data)
-	Sys_Error("%s: allready allocated", __func__);
+	Sys_Error("%s: already allocated", __func__);
 
     if (size <= 0)
 	Sys_Error("%s: size %i", __func__, size);
 
-    size = (size + sizeof(cache_system_t) + 15) & ~15;
+    size = (size + pad + sizeof(cache_system_t) + 15) & ~15;
 
     /* find memory for it */
     while (1) {
 	cs = Cache_TryAlloc(size, false);
 	if (cs) {
 	    strncpy(cs->name, name, sizeof(cs->name) - 1);
-	    c->data = (void *)(cs + 1);
 	    cs->user = c;
+	    c->pad = pad;
+	    c->data = Cache_Data(cs);
+	    c->destructor = NULL;
 	    break;
 	}
-	/* free the least recently used cahedat */
+	/* free the least recently used cache data */
 	if (cache_head.lru_prev == &cache_head)
 	    Sys_Error("%s: out of memory", __func__);
 	/* not enough memory at all */
@@ -915,8 +1178,42 @@ Cache_Alloc(cache_user_t *c, int size, char *name)
     return Cache_Check(c);
 }
 
+static void
+Cache_f(void)
+{
+    if (Cmd_Argc() == 2) {
+	if (!strcmp(Cmd_Argv(1), "print")) {
+	    Cache_Print();
+	    return;
+	}
+	if (!strcmp(Cmd_Argv(1), "flush")) {
+	    Cache_Flush();
+	    return;
+	}
+    }
+    Con_Printf("Usage: cache print|flush\n");
+}
+
 /* ========================================================================= */
 
+size_t
+Memory_GetSize(void)
+{
+    int parm;
+
+    /* The -mem parameter gives heap size in megabytes */
+    parm = COM_CheckParm("-mem");
+    if (parm && com_argc > parm + 1)
+	return (size_t)atoi(com_argv[parm + 1]) << 20;
+
+    /* The -heapsize parameter gives the heap size in kilobytes */
+    parm = COM_CheckParm("-heapsize");
+    if (parm && com_argc > parm + 1)
+	return (size_t)atoi(com_argv[parm + 1]) << 10;
+
+    /* Default memory is now 128MB */
+    return (size_t)128 << 20;
+}
 
 /*
  * ========================
@@ -929,10 +1226,11 @@ Memory_Init(void *buf, int size)
     int p;
     int zonesize = DYNAMIC_SIZE;
 
-    hunk_base = buf;
-    hunk_size = size;
-    hunk_low_used = 0;
-    hunk_high_used = 0;
+    hunkstate.base = buf;
+    hunkstate.size = size;
+    hunkstate.lowbytes = 0;
+    hunkstate.highbytes = 0;
+    hunkstate.tempmark = 0;
 
     Cache_Init();
     p = COM_CheckParm("-zone");
@@ -948,4 +1246,7 @@ Memory_Init(void *buf, int size)
 
     /* Needs to be added after the zone init... */
     Cmd_AddCommand("flush", Cache_Flush);
+    Cmd_AddCommand("hunk", Hunk_f);
+    Cmd_AddCommand("zone", Z_Zone_f);
+    Cmd_AddCommand("cache", Cache_f);
 }
