@@ -23,19 +23,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "qtypes.h"
 
-/* FIXME - it's just not clean... */
-#ifdef _WIN32
-#include <windows.h>
-#include <commctrl.h>
-LONG CDAudio_MessageHandler(HWND hWnd, UINT uMsg, WPARAM wParam,
-			    LPARAM lParam);
-#endif
-
 int CDAudio_Init(void);
-void CDAudio_Play(byte, qboolean);
+void CDAudio_Play(byte track, qboolean looping);
 void CDAudio_Stop(void);
 void CDAudio_Pause(void);
 void CDAudio_Resume(void);
+void CDAudio_InvalidateDisk(void);
 void CDAudio_Shutdown(void);
 void CDAudio_Update(void);
 
