@@ -65,17 +65,25 @@ The user can access cvars from the console in two ways:
 Cvars are restricted from having the same names as commands to keep this
 interface from being ambiguous.
 
-TYR - last argument is a callback function that is called when the value
-      of the cvar is changed.
+Callback is a function that is called when the value of the cvar is changed.
+
 */
 
 struct cvar_s;
 
 typedef void (*cvar_callback) (struct cvar_s *);
 
+/*
+ * Cvar argument completion function.
+ * Pass in the argument string
+ * Returns a string tree of possible completions
+ * Requires STree_AllocInit() prior to calling
+ */
+typedef struct stree_root *(*cvar_arg_f)(const char *);
+
 typedef struct cvar_s {
-    char *name;
-    char *string;
+    const char *name;
+    const char *string;
     qboolean archive;	// set to true to cause it to be saved to vars.rc
 
     // FIXME - obviously...
@@ -87,22 +95,15 @@ typedef struct cvar_s {
 #endif
 
     float value;
-    struct cvar_s *next; // Old way to chain them together...
-
     cvar_callback callback;
     unsigned flags;
-
-    struct completion index; /* for searching through vars and commands */
-
+    struct stree_node stree; /* string tree for cvar names */
+    cvar_arg_f completion;
 } cvar_t;
 
 #define CVAR_DEVELOPER (1U << 0) /* can't set during normal play */
 #define CVAR_OBSOLETE  (1U << 1) /* cvar has no effect; basically removed */
 
-/* Get a pointer to the cvar from the completion pointer */
-#define cvar_for_completion(ptr) \
-    ((cvar *)((char *)(ptr)-(unsigned long)(&((cvar *)0)->index)))
-
 /*
  * register a cvar that already has the name, string, and optionally the
  * archive elements set.
@@ -110,23 +111,16 @@ typedef struct cvar_s {
 void Cvar_RegisterVariable(cvar_t *variable);
 
 /* equivelant to "<name> <variable>" typed at the console */
-void Cvar_Set(char *var_name, char *value);
+void Cvar_Set(const char *var_name, const char *value);
 
 /* expands value to a string and calls Cvar_Set */
-void Cvar_SetValue(char *var_name, float value);
+void Cvar_SetValue(const char *var_name, float value);
 
 /* returns 0 if not defined or non numeric */
-float Cvar_VariableValue(char *var_name);
+float Cvar_VariableValue(const char *var_name);
 
 /* returns an empty string if not defined */
-char *Cvar_VariableString(char *var_name);
-
-/*
- * attempts to match a partial variable name for command line completion
- * returns NULL if nothing fits
- * FIXME - use common completion functions (shell.h)
- */
-char *Cvar_CompleteVariable(char *partial);
+const char *Cvar_VariableString(const char *var_name);
 
 /*
  * called by Cmd_ExecuteString when Cmd_Argv(0) doesn't match a known command.
@@ -142,8 +136,15 @@ qboolean Cvar_Command(void);
 void Cvar_WriteVariables(FILE *f);
 
 /* */
-cvar_t *Cvar_FindVar(char *var_name);
+cvar_t *Cvar_FindVar(const char *var_name);
 
-extern cvar_t *cvar_vars;
+char *Cvar_ArgComplete(const char *name, const char *buf);
+struct stree_root *Cvar_ArgCompletions(const char *name, const char *buf);
+
+# ifdef NQ_HACK
+cvar_t *Cvar_NextServerVar(const char *var_name);
+#endif
+
+struct stree_root cvar_tree;
 
 #endif /* CVAR_H */
