@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define VID_H
 
 #include "qtypes.h"
+#include "keys.h"
 
 #define VID_CBITS	6
 #define VID_GRADES	(1 << VID_CBITS)
@@ -40,41 +41,92 @@ typedef struct {
     pixel_t *colormap;		// 256 * VID_GRADES size
     unsigned short *colormap16;	// 256 * VID_GRADES size
     int fullbright;		// index of first fullbright color
-    unsigned rowbytes;		// may be > width if displayed in a window
-    unsigned width;
-    unsigned height;
+    int rowbytes;		// may be > width if displayed in a window
+    int width;
+    int height;
     float aspect;		// width / height -- < 0 is taller than wide
     int numpages;
     int recalc_refdef;		// if true, recalc vid-based stuff
     pixel_t *conbuffer;
     int conrowbytes;
-    unsigned conwidth;
-    unsigned conheight;
+    int conwidth;
+    int conheight;
     int maxwarpwidth;
     int maxwarpheight;
-    pixel_t *direct;		// direct drawing to framebuffer, if not
-    //  NULL
+    pixel_t *direct;		// direct drawing to framebuffer, if not NULL
 } viddef_t;
 
 extern viddef_t vid;		// global video state
 extern unsigned short d_8to16table[256];
 extern unsigned d_8to24table[256];
-extern void (*vid_menudrawfn) (void);
-extern void (*vid_menukeyfn) (int key);
 
-void VID_SetPalette(unsigned char *palette);
+/*
+ * ------------------------------------------------------------------------
+ * VIDEO MODES
+ * (very ugly, cleanup pending...)
+ * ------------------------------------------------------------------------
+ */
+
+typedef struct {
+    int modenum;
+    int width;
+    int height;
+    int bpp;
+    int refresh;
+    byte driverdata[8];	/* Allow drivers to stuff some data */
+} qvidmode_t;
+
+/*
+ * TODO ~ Have the vid driver allocate modelist dynamically
+ */
+#define MAX_MODE_LIST 600
+extern qvidmode_t modelist[MAX_MODE_LIST];
+extern qvidmode_t badmode;
+
+extern int nummodes;
+extern int vid_modenum;
+
+/* FIXME - vid mode testing */
+extern int vid_testingmode;
+extern int vid_realmode;
+extern double vid_testendtime;
+
+#define VID_MODE_NONE               (-1)
+#define VID_MODE_WINDOWED           0
+
+void VID_InitModeCvars(void);
+void VID_SortModeList(qvidmode_t *modelist, int nummodes);
+const qvidmode_t *VID_GetCmdlineMode(void);
+
+void VID_MenuDraw(void);
+void VID_MenuInitState(const qvidmode_t *mode);
+void VID_MenuKey(knum_t keynum);
+qboolean VID_SetMode(const qvidmode_t *mode, const byte *palette);
+qboolean VID_CheckAdequateMem(int width, int height);
+void VID_NumModes_f(void);
+void VID_DescribeModes_f(void);
+void VID_DescribeMode_f(void);
+void VID_DescribeCurrentMode_f(void);
+
+extern void (*vid_menudrawfn)(void);
+extern void (*vid_menukeyfn)(knum_t keynum);
+
+/* ------------------------------------------------------------------------ */
+
+void VID_SetPalette(const byte *palette);
 
 // called at startup and after any gamma correction
 
-void VID_ShiftPalette(unsigned char *palette);
+void VID_ShiftPalette(const byte *palette);
 
 // called for bonus and pain flashes, and for underwater color changes
 
+extern unsigned short ramps[3][256];
 extern void (*VID_SetGammaRamp)(unsigned short ramp[3][256]);
 
 // called to set hardware gamma (if available - primarily for OpenGL renderer)
 
-void VID_Init(unsigned char *palette);
+void VID_Init(const byte *palette);
 
 // Called at startup to set up translation tables, takes 256 8 bit RGB values
 // the palette data will go away after the call, so it must be copied off if
@@ -88,16 +140,9 @@ void VID_Update(vrect_t *rects);
 
 // flushes the given rectangles from the view buffer to the screen
 
-void VID_HandlePause(qboolean pause);
-
-// called only on Win32, when pause happens, so the mouse can be released
 void VID_LockBuffer(void);
 void VID_UnlockBuffer(void);
 
-#ifdef GLQUAKE
-qboolean VID_Is8bit(void);
-#endif
-
 qboolean VID_IsFullScreen(void);
 
 #endif /* VID_H */
